#ifndef CORAL_CORALBASE_DATE_H
#define CORAL_CORALBASE_DATE_H

#include "SealBase/Time.h"

namespace coral
{
  /**
     @class Date Date.h CoralBase/Date.h

     A class defining the ANSI DATE type implemented on the seal::Time class
   */
  class Date
  {
  public:
    /// Default constructor
    Date();

    /// Constructor
    Date( int year,
          int month,
          int day );

    /// Destructor
    ~Date();

    /// Constructor from a seal time
    explicit Date( const seal::Time& sealTime );

    /// Copy constructor
    Date( const Date& rhs );

    /// Assignment operator
    Date& operator=( const Date& rhs );

    /// Equal operator
    bool operator==( const Date& rhs ) const;

    /// Comparison operator
    bool operator!=( const Date& rhs ) const;

    /// Returns the year
    int year() const;

    /// Returns the month [1-12]
    int month() const;

    /// Returns the day [1-31]
    int day() const;

    /// Returns the underlying seal::Time object
    const seal::Time& time() const;

  public:
    static Date today();

  private:
    /// The actual time object
    seal::Time m_time;
  };
}




/// Inline methods
inline
coral::Date::Date():
  m_time()
{}

inline
coral::Date::Date( int year,
                   int month,
                   int day ):
  m_time( year, month-1, day, 0, 0, 0, 0 )
{}

inline
coral::Date::~Date()
{}

inline
coral::Date::Date( const seal::Time& sealTime ):
  m_time( sealTime )
{}

inline
coral::Date::Date( const coral::Date& rhs ):
  m_time( rhs.m_time )
{}

inline coral::Date&
coral::Date::operator=( const coral::Date& rhs )
{
  m_time = rhs.m_time;
  return *this;
}

inline int
coral::Date::year() const
{
  return m_time.year( true );
}

inline int
coral::Date::month() const
{
  return m_time.month( true ) + 1;
}

inline int
coral::Date::day() const
{
  return m_time.day( true );
}


inline const seal::Time&
coral::Date::time() const
{
  return m_time;
}


inline bool
coral::Date::operator==( const coral::Date& rhs ) const
{
  return ( this->year() == rhs.year() &&
           this->month() == rhs.month() &&
           this->day() == rhs.day() );
}


inline bool
coral::Date::operator!=( const coral::Date& rhs ) const
{
  return ( this->year() != rhs.year() ||
           this->month() != rhs.month() ||
           this->day() != rhs.day() );
}


#endif
