#include <iostream>
#include <exception>
#include "CoralCommon/URIParser.h"
#include "CoralBase/Exception.h"

void testParser( coral::URIParser& parser,
                 const std::string& uri )
{
  parser.setURI( uri );
  std::cout << "Results for \"" + parser.getURI() + "\" :" << std::endl;
  std::cout << "   Technology             : \"" << parser.technology() << "\"" << std::endl;
  std::cout << "   Protocol               : \"" << parser.protocol() << "\"" << std::endl;
  std::cout << "   Host name              : \"" << parser.hostName() << "\"" << std::endl;
  std::cout << "   Port number            : " << parser.portNumber() << std::endl;
  std::cout << "   Database (schema) name : \"" << parser.databaseOrSchemaName() << "\"" << std::endl;
}


int main( int, char** )
{
  try {
    coral::URIParser parser;
    testParser( parser, "oracle://dbhost/D9/user" );
    testParser( parser, "oracle://dbhost/user" );
    testParser( parser, "mysql://myhost/userdb" );
    testParser( parser, "mysql://myhost:1105/userdb" );
    testParser( parser, "xml_file:mylocalFile.xml" );
    testParser( parser, "xml_file:/absolutePath.xml" );
    testParser( parser, "xml_file:///absolutePath.xml" );
    testParser( parser, "xml_http://myhost/absolutePath.xml" );
  }
  catch( coral::Exception& e ) {
    std::cerr << "CORAL exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( std::exception& e ) {
    std::cerr << "C++ exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Funny error ..." << std::endl;
    return 1;
  }
  std::cout << "[OVAL] Success" << std::endl;
  return 0;
}
