'''
exporter(sourceschema,destschema)
Input parameter sourceSchema : a schema object, obtained by the sessionproxy object
Input parameter destSchema : a schema object, obtained by the sessionproxy object
Input parameter rowCachesize :  the number of rows to be cached at the client side, default value =100
Output paramter : the exporter object
'''

import os
import coral
import time
import math
from multivaluedict import mseqdict
from schemautils.listobjects import listobjects,listschema,listtables,listtableset

class exporter:
 "exporter class for CoralTools"
 m_sourceSchema = 0
 m_destSchema = 0
 m_rowCachesize = 100

 def __init__( self,sourceSchema,destSchema,rowCachesize=100 ):
  try:

   self.m_sourceSchema = sourceSchema
   self.m_destSchema = destSchema
   self.m_rowCachesize = rowCachesize

  except Exception, e:
   raise Exception("Error in Initializer: " + str(e)) 

#Copies the schema objects from source to destination, without copying data. 
 def copyschema(self ):
  try:

   listsourceTable=listschema( self.m_sourceSchema )
   listdestTable=listobjects( self.m_destSchema )
   self._checktable(listsourceTable,listdestTable)

   for key,value in listsourceTable.items():
    iTable= key
    self._copytablelayout(iTable)

   print "copyschema SUCCESS"

   return True

  except Exception, e:
   raise Exception ("Error in copyschema method: " + str(e))
   return False

#Copies the schema objects from source to destination, including data. 
 def copydata(self ):
  try:

   listsourceTable=listschema( self.m_sourceSchema )
   listdestTable=listobjects( self.m_destSchema )
   self._checktable(listsourceTable,listdestTable)

   selectionclause=""
   selectionparameters=coral.AttributeList()

   for key,value in listsourceTable.items():
    iTable= key
    self._copytablelayout(iTable)
    self._copydatalayout(iTable,selectionclause,selectionparameters)

   print "copydata SUCCESS"
   return True

  except Exception, e:
   raise Exception ("Error in copydata method: " + str(e))
   return False

#Copies the specified table schema without copying data. 
 def copytableschema(self,tablename ):
  try:
   iTable=""
   listsourceTable=listtables( self.m_sourceSchema,tablename )
   listdestTable=listtables( self.m_destSchema,tablename )
   self._checktable(listsourceTable,listdestTable)

   for key,value in listsourceTable.items():
    iTable= key
    self._copytablelayout(iTable)

   print "copytableschema SUCCESS"
   return True

  except Exception, e:
   raise Exception ("Error in copytableschema method: " + str(e)+" : "+iTable)
   return False

#Copies the specified table schema including data.  
 def copytabledata(self,tablename,selectionclause,selectionparameters ):
  try:

   iTable=""
   listsourceTable=listtables( self.m_sourceSchema,tablename )
   listdestTable=listtables( self.m_destSchema,tablename )

   for key,value in listsourceTable.items():
    iTable= key
    tableexists = self._checkdata(iTable,listdestTable)

    if not tableexists: 
       self._copytablelayout(iTable)

    self._copydatalayout(iTable,selectionclause,selectionparameters)

   print "copytabledata SUCCESS"
   return True

  except Exception, e:
   raise Exception  ("Error in copytabledata method: " + str(e)+" : " + iTable)
   return False

#Copies the specified list of tables ordered by hierarchy without copying data.
 def copytablelistschema(self,tableset ):
  try:

   iTable=""
   listsourceTable=listtableset( self.m_sourceSchema,tableset )
   listdestTable=listtableset( self.m_destSchema,tableset )
   self._checktable(listsourceTable,listdestTable)

   for key,value in listsourceTable.items():
       iTable= key
       self._copytablelayout(iTable)

   print "copytablelistschema SUCCESS"
   return True

  except Exception, e:
   raise Exception ("Error in copytablelistschema method: " + str(e)+" : "+iTable)
   return False

#Copies the specified list of tables ordered by hierarchy including data.
 def copytablelistdata(self,tablelist):
  try:

   iTable=""
   tableset=[]
   for table in tablelist:
       i=0
       for parameter in  table:
        if i==0:
           tableset.append(parameter)
        i=i+1

   listsourceTable=listtableset( self.m_sourceSchema,tableset )
   listdestTable=listtableset( self.m_destSchema,tableset )

   for key,value in listsourceTable.items():
       iTable= key
       selectionclause=""
       selectionparameters=coral.AttributeList()
       for table in tablelist:
           i=0
           for parameter in  table:
                 if i==0:
                    table=parameter
                 if table==iTable:
                    if i==1:
                       selectionclause = parameter
                    if i==2:
                       selectionparameters = parameter
                 i=i+1

       tableexists = self._checkdata(iTable,listdestTable)

       if not tableexists:
           self._copytablelayout(iTable)

       self._copydatalayout(iTable,selectionclause,selectionparameters)

   print "copytablelistdata SUCCESS"
   return True

  except Exception, e:
   raise Exception ("Error in copytablelistdata method: " + str(e) + " : "+ iTable)
   return False

#Copies the schema objects from source to destination 
 def _copytablelayout(self,iTable ):
  try:

    frmSchmaTblDsc = self.m_sourceSchema.tableHandle( iTable ).description()
    description = coral.TableDescription(self.m_sourceSchema.tableHandle( iTable ).description())
    description.setName( iTable )

    numberOfIndices = frmSchmaTblDsc.numberOfIndices()
    for ii in range(0, numberOfIndices ):
       index = frmSchmaTblDsc.index( ii )
       columnNames = index.columnNames()
       description.createIndex( index.name(),columnNames) 

    numberOfForeignKeys = frmSchmaTblDsc.numberOfForeignKeys()
    for i in range(0, numberOfForeignKeys):
       foreignKey = frmSchmaTblDsc.foreignKey( i )
       columnNames = foreignKey.columnNames()
       columnNamesR = foreignKey.referencedColumnNames()
       description.createForeignKey( foreignKey.name(), columnNames, foreignKey.referencedTableName(),columnNamesR )
 
    table = self.m_destSchema.createTable( description )
    return True

  except Exception, e:
   raise Exception (" " + str(e))
   return False

#Copies the data from source to destination 
 def _copydatalayout(self,iTable,selectionclause,selectionparameters ):
  try:

    data=coral.AttributeList()
    sourceEditor = self.m_sourceSchema.tableHandle( iTable ).dataEditor()
    destEditor = self.m_destSchema.tableHandle( iTable ).dataEditor()

    sourceEditor.rowBuffer(data)
    sourcequery = self.m_sourceSchema.tableHandle( iTable ).newQuery()
    sourcequery.setCondition(selectionclause,selectionparameters)
    sourcequery.setRowCacheSize(self.m_rowCachesize)
    sourcequery.defineOutput(data)

    bulkOperation = destEditor.bulkInsert(data,self.m_rowCachesize )

    cursor=sourcequery.execute()
    for row in cursor:
        bulkOperation.processNextIteration()

    bulkOperation.flush()
    del bulkOperation
    del sourcequery

    return True

  except Exception, e:
   raise Exception (" " + str(e))
   return False

 #Checks if table exists in destination schema
 def _checktable(self,listsourceTable,listdestTable ):
  try:

   for key,value in listsourceTable.items():
     table=key
     for key,value in listdestTable.items():
        if key==table:
          raise Exception( "Table exists in Destination Schema : " )

   return True

  except Exception, e:
   raise Exception (" " + str(e) + table)
   return False

#Checks if data exists in the table in destination schema
 def _checkdata(self,iTable,listdestTable ):
  try:

   foundtable=False
   founddata=False
   for key,value in listdestTable.items():
      if key==iTable:
         counter=0
         # Run a query on the destination table
         query = self.m_destSchema.tableHandle( iTable ).newQuery()
         cursor = query.execute()
         foundtable=True
         for row in cursor:
            counter=counter+1
         del query
         if (counter > 0):
            founddata=True

   return foundtable

  except Exception, e:
   raise Exception (" " + str(e) + iTable)
   return False

 def __del__( self ):
     print ""
