#Coral package:CoralTools 
import os
import coral
from schemautils.exporter import exporter

try: 
  #Initialize Connection Service
  svc = coral.ConnectionService()

  #open session proxy for source schema providing logical service name & access mode
  session1 = svc.connect( 'write_test', accessMode = coral.access_Update )
  transaction1 = session1.transaction()

  #open session proxy for destination schema providing logical service name & access mode
  session2 = svc.connect( 'coral_test', accessMode = coral.access_Update )
  transaction2 = session2.transaction()
  
  sourceSchema=session1.nominalSchema()
  destSchema=session2.nominalSchema()
  rowCachesize=1000
  exp=exporter( sourceSchema,destSchema,rowCachesize )

  transaction1.start()
  transaction2.start()

  try: 
    print "copyschema() : Copies the schema objects from source to destination, without copying data."
    exp.copyschema() 
    transaction2.commit()
    print "Tables created" 
  except Exception, e:
    transaction2.rollback()
    print "Test Failed" 
    print str(e)

  try: 
    print "copydata() : Copies the schema objects from source to destination, including data copy."
    exp.copydata() 
    transaction2.commit()
    print "Data copied"
  except Exception, e:
    transaction2.rollback()
    print "Test Failed" 
    print str(e)

  try: 
    print "copytableschema(tablename) : Copies the specified table schema without copying data."
    tablename = "T3"
    exp.copytableschema(tablename) 
    transaction2.commit()
    print "Tables created" 
  except Exception, e:
    transaction2.rollback()
    print "Test Failed" 
    print str(e)

  try: 
    print "copytabledata(tablename,selectionclause,selectionparameters) : Copies the specified table schema, including data copy."
    tablename = "T1"
    selectionclause= "id >= :idmin and id < :idmax "
    selectionparameters = coral.AttributeList()
    selectionparameters.extend( "idmin","int")
    selectionparameters.extend( "idmax","int")
    selectionparameters["idmin"].setData(0)
    selectionparameters["idmax"].setData(25)
    exp.copytabledata(tablename,selectionclause,selectionparameters) 
    transaction2.commit()
    print "Data copied"
  except Exception, e:
    transaction2.rollback()
    print "Test Failed" 
    print str(e)

  try: 
    print "copytablelistschema(tablelist) : Copies the specified list of tables ordered by hierarchy without copying data."
    tableset = ['T3','T1','T2']
    exp.copytablelistschema(tableset) 
    transaction2.commit()
    print "Tables created" 
  except Exception, e:
    transaction2.rollback()
    print "Test Failed" 
    print str(e)

  try: 
    print "copytablelistdata(tablelist) : Copies the specified list of tables ordered by hierarchy, including data copy."
    table1 = "T3"
    selectionclause1= "id >= 4 and id < 8 "

    table2 = "T2"
    selectionclause2= "id >= :idmin and id < :idmax "
    selectionparameters2 = coral.AttributeList()
    selectionparameters2.extend( "idmin","int")
    selectionparameters2.extend( "idmax","int")
    selectionparameters2["idmin"].setData(0)
    selectionparameters2["idmax"].setData(34)

    table3 = "T1"

    tablelist1 = [table1,selectionclause1]
    tablelist2 = [table2,selectionclause2,selectionparameters2]
    tablelist3 = [table3]

    tablelist = [tablelist1,tablelist2,tablelist3]
    exp.copytablelistdata(tablelist) 
    transaction2.commit()
    print "Data copied " 
  except Exception, e:
    transaction2.rollback()
    print "Test Failed" 
    print str(e)

  transaction1.commit()
  del session2
  del session1

except Exception, e:
 print "Test FAILED"
 print str(e)


