#include "EnvironmentAuthenticationService.h"
#include "CoralCommon/AuthenticationCredentials.h"
#include "RelationalAccess/AuthenticationServiceException.h"
#include "SealKernel/MessageStream.h"
#include <cstdlib>


DEFINE_SEAL_COMPONENT ( coral::EnvironmentAuthenticationService::EnvironmentAuthenticationService, "CORAL/Services/EnvironmentAuthenticationService")


coral::EnvironmentAuthenticationService::EnvironmentAuthenticationService::EnvironmentAuthenticationService( seal::Context* c )
  : seal::Service( c, classContextKey() ),
    m_credentials( 0 )
{}


coral::EnvironmentAuthenticationService::EnvironmentAuthenticationService::EnvironmentAuthenticationService( seal::Context* c,
                                                                                                             const std::string& key )
  : seal::Service( c, key ),
    m_credentials( 0 )
{}


coral::EnvironmentAuthenticationService::EnvironmentAuthenticationService::~EnvironmentAuthenticationService()
{
  if ( m_credentials ) delete m_credentials;
}


const coral::IAuthenticationCredentials&
coral::EnvironmentAuthenticationService::EnvironmentAuthenticationService::credentials( const std::string& ) const
{
  if ( m_credentials == 0 ) {
    m_credentials = new coral::AuthenticationCredentials( this->name() );

    char* userName = ::getenv( "CORAL_AUTH_USER" );
    if ( userName != 0 ) {
      m_credentials->registerItem( "user", std::string( userName ) );
    }
    else {
      delete m_credentials;
      m_credentials = 0;
      seal::MessageStream log( const_cast< coral::EnvironmentAuthenticationService::EnvironmentAuthenticationService* >( this ), this->name() );
      log << seal::Msg::Error << "The CORAL_AUTH_USER environment variable is not set!" << seal::flush;
      throw coral::AuthenticationServiceException( "The CORAL_AUTH_USER environment variable is not set.",
                                                   "IAuthenticationService::credentials",
                                                   this->name() );
    }
    char* password = ::getenv( "CORAL_AUTH_PASSWORD" );
    if ( password != 0 ) {
      m_credentials->registerItem( "password", std::string( password ) );
    }
    else {
      delete m_credentials;
      m_credentials = 0;
      seal::MessageStream log( const_cast< coral::EnvironmentAuthenticationService::EnvironmentAuthenticationService* >( this ), this->name() );
      log << seal::Msg::Error << "The CORAL_AUTH_PASSWORD environment variable is not set!" << seal::flush;
      throw coral::AuthenticationServiceException( "The CORAL_AUTH_PASSWORD environment variable is not set.",
                                                   "IAuthenticationService::credentials",
                                                   this->name() );
    }
  }

  return *m_credentials;
}
