#include <iostream>
#include <vector>
#include <stdexcept>
#include <cstdlib>
#include "PluginManager/PluginManager.h"
#include "SealKernel/ComponentLoader.h"
#include "CoralBase/Exception.h"
#include "SealKernel/Context.h"
#include "RelationalAccess/IAuthenticationService.h"
#include "RelationalAccess/IAuthenticationCredentials.h"

int main( int, char** )
{
  try {
    ::putenv( "CORAL_AUTH_USER=dbuser" );
    ::putenv( "CORAL_AUTH_PASSWORD=dbpasswd" );

    std::vector< std::string > connections;
    connections.push_back( "oracle://host1/USER" );
    connections.push_back( "oracle://host2/USER" );

    seal::PluginManager* pm = seal::PluginManager::get();
    pm->initialise();

    seal::Handle< seal::Context > context = new seal::Context();
    seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( context.get() );

    loader->load( "CORAL/Services/EnvironmentAuthenticationService" );

    std::vector< seal::IHandle<coral::IAuthenticationService> > v_authSvc;
    context->query( v_authSvc );
    if ( ! v_authSvc.empty() ) {
      seal::IHandle<coral::IAuthenticationService>& authSvc = v_authSvc.front();

      for ( std::vector< std::string >::const_iterator iConnection = connections.begin();
            iConnection != connections.end(); ++iConnection ) {
        std::cout << "Connection \"" << *iConnection << "\" :" << std::endl;
        const coral::IAuthenticationCredentials& credentials = authSvc->credentials( *iConnection );
        int numberOfItems = credentials.numberOfItems();
        for ( int i = 0; i < numberOfItems; ++i ) {
          std::string itemName = credentials.itemName( i );
          std::cout << "    " << itemName << " : " << credentials.valueForItem( itemName ) << std::endl;
        }
      }
    }
    else {
      throw std::runtime_error("Could not retrieve an authentication service.");
    }

    std::cout << "Exiting..." << std::endl;
  }
  catch ( coral::Exception& se ) {
    std::cerr << "CORAL Exception : " << se.what() << std::endl;
    return 1;
  }
  catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
    return 1;
  }
  std::cout << "[OVAL] Success" << std::endl;
  return 0;
}
