#include "DomainProperties.h"

#include "Domain.h"

#include "RelationalAccess/IConnectionService.h"

coral::FrontierAccess::DomainProperties::DomainProperties( coral::FrontierAccess::Domain* service ):
  m_service( service ),
  m_tableSpaceForTables(""),
  m_tableSpaceForIndices(""),
  m_tableSpaceForLobs(""),
  m_lobChunkSize(0)
{}


coral::FrontierAccess::DomainProperties::~DomainProperties()
{}


void
coral::FrontierAccess::DomainProperties::setTableSpaceForTables( const std::string& tableSpace )
{
  m_tableSpaceForTables = tableSpace;
}
      

void
coral::FrontierAccess::DomainProperties::setTableSpaceForIndices( const std::string& tableSpace )
{
  m_tableSpaceForIndices = tableSpace;
}


void
coral::FrontierAccess::DomainProperties::setTableSpaceForLobs( const std::string& tableSpace )
{
  m_tableSpaceForLobs = tableSpace;
}


void
coral::FrontierAccess::DomainProperties::setLobChunkSize( int lobChunkSize )
{
  m_lobChunkSize = lobChunkSize;
}


seal::Service* coral::FrontierAccess::DomainProperties::service() const
{
  return dynamic_cast<seal::Service*>( this->m_service );
}


std::string
coral::FrontierAccess::DomainProperties::tableSpaceForTables() const
{
  return m_tableSpaceForTables;
}


std::string
coral::FrontierAccess::DomainProperties::tableSpaceForIndices() const
{
  return m_tableSpaceForIndices;
}


std::string
coral::FrontierAccess::DomainProperties::tableSpaceForLobs() const
{
  return m_tableSpaceForIndices;
}


int
coral::FrontierAccess::DomainProperties::lobChunkSize() const
{
  return m_lobChunkSize;
}

const coral::IWebCacheControl& coral::FrontierAccess::DomainProperties::cacheControl() const
{
  seal::IHandle< coral::IConnectionService > consvc = this->connectionService();

  if( ! consvc )
    throw std::runtime_error( "Did not find coral::ConnectionService " );

  return consvc->webCacheControl();
}

seal::IHandle< coral::IConnectionService > coral::FrontierAccess::DomainProperties::connectionService() const
{
  std::vector<seal::IHandle<coral::IConnectionService > > cv;

  this->m_service->localContext()->query( cv );

  if( cv.empty() )
    return seal::IHandle<coral::IConnectionService >( seal::Handle<seal::Service>(0) );

  seal::IHandle<coral::IConnectionService > csvc = cv[0];
  
  return csvc;
}
