// $Id: SessionProperties.cpp,v 1.7 2007/01/26 10:24:32 rado Exp $

#include "DomainProperties.h"
#include "SessionProperties.h"
#include "TypeConverter.h"

#include "RelationalAccess/ISession.h"
#include "RelationalAccess/ITransaction.h"

#include "SealKernel/Service.h"

#include <stdexcept>
#include <locale>

coral::FrontierAccess::SessionProperties::SessionProperties( const DomainProperties& domainProperties,
                                                             const std::string& connectionString,
                                                             frontier::Connection& fconnection,
                                                             boost::mutex& flock,
                                                             const std::string& schemaName,
                                                             coral::ITypeConverter& converter,
                                                             coral::ISession& session )
  : m_domainProperties( domainProperties )
  , m_connectionString( connectionString )
  , m_connectionURL( connectionString )
  , m_connection( fconnection )
  , m_session( session )
  , m_typeConverter( converter )
  , m_monitoringService( 0 )
  , m_serverVersion( 0 )
  , m_schemaName( "" )
  , m_lock( flock )
{
  if( ! schemaName.empty() )
    setSchemaName( schemaName );
}

coral::FrontierAccess::SessionProperties::~SessionProperties()
{
}

void coral::FrontierAccess::SessionProperties::setServerVersion( int majorServerVersion )
{
  m_serverVersion = majorServerVersion;
}

coral::ITypeConverter& coral::FrontierAccess::SessionProperties::typeConverter()
{
  return m_typeConverter;
}

const coral::ITypeConverter& coral::FrontierAccess::SessionProperties::typeConverter() const
{
  return m_typeConverter;
}

bool coral::FrontierAccess::SessionProperties::isTransactionActive() const
{
  return m_session.transaction().isActive();
}

coral::ISchema& coral::FrontierAccess::SessionProperties::schema() const
{
  return m_session.nominalSchema();
}

std::string coral::FrontierAccess::SessionProperties::schemaName() const
{
  return m_schemaName;
}

void coral::FrontierAccess::SessionProperties::setSchemaName( const std::string& newSchemaName )
{
  m_schemaName           = "";
  std::string tempSchema = newSchemaName;
  for ( std::string::size_type i = 0; i < tempSchema.size(); ++i )
  {
    m_schemaName += std::toupper( tempSchema[i], std::locale::classic() );
  }
}

seal::IHandle< coral::IConnectionService > coral::FrontierAccess::SessionProperties::connectionService() const
{
  std::vector<seal::IHandle<coral::IConnectionService > > cv;

  m_domainProperties.service()->localContext()->query( cv );

  if( cv.empty() )
//     throw std::runtime_error( "Did not find coral::ConnectionService " );
    return seal::IHandle<coral::IConnectionService >( seal::Handle<seal::Service>(0) );

  seal::IHandle<coral::IConnectionService > csvc = cv[0];
  
  return csvc;
}
