#include "TestBase.h"

#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/RelationalServiceException.h"
#include "RelationalAccess/IConnectionService.h"

#include "PluginManager/PluginManager.h"

#include "SealKernel/IMessageService.h"
#include "SealKernel/Context.h"
#include "SealKernel/ComponentLoader.h"

#include <stdexcept>

TestBase::TestBase()
  : m_context( new seal::Context )
{
  seal::PluginManager* pm = seal::PluginManager::get();
  pm->initialise();
  seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( m_context.get() );
  loader->load( "CORAL/RelationalPlugins/frontier" );
  loader->load( "CORAL/RelationalPlugins/oracle" );
  if( getenv( "CORAL_AUTH_PATH" ) )
    loader->load( "CORAL/Services/XMLAuthenticationService" );
  else
    loader->load( "CORAL/Services/EnvironmentAuthenticationService" );
  loader->load( "SEAL/Services/MessageService" );

  std::vector< seal::Handle<seal::IMessageService> > v_msgSvc;
  m_context->query( v_msgSvc );
  if ( ! v_msgSvc.empty() ) {
    seal::Handle<seal::IMessageService>& msgSvc = v_msgSvc.front();
    msgSvc->setOutputLevel( seal::Msg::Verbose);
  }
  
  loader->load( "CORAL/Services/ConnectionService" );
}

TestBase::~TestBase()
{
}

coral::IConnection* connection = 0;

coral::ISession* TestBase::connect( const std::string& connectionString, const std::string& userName, const std::string& password, coral::AccessMode mode  )
{
  std::pair<std::string,std::string> cs;
  
  if( connectionString.find( "oracle" ) != std::string::npos )
  {
    seal::IHandle<coral::IRelationalDomain> iHandle = m_context->query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/oracle" );

    if ( ! iHandle )
    {
      throw coral::NonExistingDomainException( "oracle" );
    }

    cs = iHandle->decodeUserConnectionString( connectionString );
    
    connection = iHandle->newConnection( cs.first );
  }
  else if( connectionString.find( "frontier" ) != std::string::npos )
  {
    seal::IHandle<coral::IRelationalDomain> iHandle = m_context->query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/frontier" );

    if ( ! iHandle )
    {
      throw coral::NonExistingDomainException( "frontier" );
    }

    cs = iHandle->decodeUserConnectionString( connectionString );
    
    connection = iHandle->newConnection( cs.first );
  }

  connection->connect();
  
  coral::ISession* session = connection->newSession( cs.second, mode );
  
  if ( session )
  {
    session->startUserSession( userName, password );
  }

  return session;
}

void TestBase::disconnect()
{
  if( connection->isConnected() )
    connection->disconnect();

  delete connection;
  connection = 0;
}

coral::ITypeConverter& TestBase::typeConverter() const 
{
  return connection->typeConverter();
}

seal::IHandle<coral::IConnectionService> TestBase::connectionService()
{
  std::vector< seal::IHandle<coral::IConnectionService> > v_svc;

  m_context->query( v_svc );

  if ( v_svc.empty() )
  {
    throw std::runtime_error( "Could not locate the connection service" );
  }

  return v_svc.front();
}
