#include <iostream>
#include <sstream>
#include <stdexcept>
#include "PluginManager/PluginManager.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/Context.h"
#include "SealKernel/Service.h"
#include "SealKernel/PropertyManager.h"
#include "SealKernel/IMessageService.h"
#include "CoralBase/Exception.h"
#include "RelationalAccess/ILookupService.h"
#include "RelationalAccess/IAuthenticationService.h"
#include "RelationalAccess/IAuthenticationCredentials.h"
#include "RelationalAccess/IDatabaseServiceSet.h"
#include "RelationalAccess/IDatabaseServiceDescription.h"


void reportReplica( const coral::IDatabaseServiceDescription& replica,
                    const std::string& logicalName )
{
  std::cout << "#############################################"<<std::endl;
  std::cout << logicalName << " : "
            << replica.connectionString() << ", "
            << replica.authenticationMechanism() << ", "
            << ( ( replica.accessMode() == coral::ReadOnly ) ? std::string("Read-Only") : std::string("Update") )
            << std::endl;
}


int main( int, char** )
{

  try {

    int numberOfReplicas = 0;
    seal::PluginManager* pm = seal::PluginManager::get();
    pm->initialise();

    seal::Handle< seal::Context > context = new seal::Context();
    seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( context.get() );

// Code for Seal Message Service
   // Load the SEAL Message service and set the output threshold to Debug
   loader->load( "SEAL/Services/MessageService" );
   std::vector< seal::Handle<seal::IMessageService> > v_msgSvc;
   context->query( v_msgSvc );
   if ( ! v_msgSvc.empty() ) {
     seal::Handle<seal::IMessageService>& msgSvc = v_msgSvc.front();
     msgSvc->setOutputLevel( seal::Msg::Debug);
   }
	
// Code for Seal Message Service
   loader->load( "CORAL/Services/LFCReplicaService" );
   
    std::vector< seal::IHandle<coral::ILookupService> > v_lookupSvc;
    context->query( v_lookupSvc );
    if ( ! v_lookupSvc.empty() ) {
    	const coral::ILookupService& lookupSvc = *( v_lookupSvc.front() );
    std::vector< seal::IHandle<coral::IAuthenticationService> > v_authSvc;
    context->query( v_authSvc );
    if ( ! v_authSvc.empty() ) {
    	const coral::IAuthenticationService& authenticationSvc = *( v_authSvc.front() );
       coral::IDatabaseServiceSet* dbSet = lookupSvc.lookup( "test1/ecal_calib0");
       numberOfReplicas = dbSet->numberOfReplicas();
       std::cout <<" Found ["<<numberOfReplicas<<"] replicas"<<std::endl;
       for ( int i = 0; i < numberOfReplicas; ++i ){
         reportReplica( dbSet->replica( i ), "test1/ecal_calib0" );
         if(dbSet->replica( i ).authenticationMechanism()!=""){
           std::cout <<" Default role:"<<std::endl;
           const coral::IAuthenticationCredentials& creds = authenticationSvc.credentials(dbSet->replica( i ).connectionString());
           std::cout <<" Found ["<<creds.numberOfItems()<<"] credential items"<<std::endl;
           for(int it=0;it<creds.numberOfItems();++it){
             std::cout <<"For item="<<creds.itemName(it)<<" value="<<creds.valueForItem(creds.itemName(it))<<std::endl;
           }
           std::cout <<" Role: calib_reader"<<std::endl;
           try{
             const coral::IAuthenticationCredentials& creds0 = authenticationSvc.credentials(dbSet->replica( i ).connectionString(),"calib_reader");
             std::cout <<" Found ["<<creds0.numberOfItems()<<"] credential items"<<std::endl;
             for(int it=0;it<creds0.numberOfItems();++it){
               std::cout <<"For item="<<creds0.itemName(it)<<" value="<<creds0.valueForItem(creds0.itemName(it))<<std::endl;
             }
           } catch (coral::Exception& e){
             std::cout << "Exception: "<<e.what()<<std::endl;
           }
           std::cout <<" Role: calib_writer"<<std::endl;
           try{
             const coral::IAuthenticationCredentials& creds1 = authenticationSvc.credentials(dbSet->replica( i ).connectionString(),"calib_writer");
             std::cout <<" Found ["<<creds1.numberOfItems()<<"] credential items"<<std::endl;
             for(int it=0;it<creds1.numberOfItems();++it){
               std::cout <<"For item="<<creds1.itemName(it)<<" value="<<creds1.valueForItem(creds1.itemName(it))<<std::endl;
             }
           } catch (coral::Exception& e){
             std::cout << "Exception: "<<e.what()<<std::endl;
           }
         }
         
       }
       
       delete dbSet;
       dbSet = 0;
       dbSet = lookupSvc.lookup( "test1/ecal_calib0", coral::ReadOnly);
       numberOfReplicas = dbSet->numberOfReplicas();
       std::cout <<" Found ["<<numberOfReplicas<<"] replicas"<<std::endl;
       for ( int i = 0; i < numberOfReplicas; ++i ){
         reportReplica( dbSet->replica( i ), "test1/ecal_calib0" );
         if(dbSet->replica( i ).authenticationMechanism()!=""){
           std::cout <<" Default role:"<<std::endl;
           const coral::IAuthenticationCredentials& creds = authenticationSvc.credentials(dbSet->replica( i ).connectionString());
           std::cout <<" Found ["<<creds.numberOfItems()<<"] credential items"<<std::endl;
           for(int it=0;it<creds.numberOfItems();++it){
             std::cout <<"For item="<<creds.itemName(it)<<" value="<<creds.valueForItem(creds.itemName(it))<<std::endl;
           }
           std::cout <<" Role: calib_reader"<<std::endl;
           try{
             const coral::IAuthenticationCredentials& creds0 = authenticationSvc.credentials(dbSet->replica( i ).connectionString(),"calib_reader");
             std::cout <<" Found ["<<creds0.numberOfItems()<<"] credential items"<<std::endl;
             for(int it=0;it<creds0.numberOfItems();++it){
               std::cout <<"For item="<<creds0.itemName(it)<<" value="<<creds0.valueForItem(creds0.itemName(it))<<std::endl;
             }
           } catch (coral::Exception& e){
             std::cout << "Exception: "<<e.what()<<std::endl;
           }
           std::cout <<" Role: calib_writer"<<std::endl;
           try{
             const coral::IAuthenticationCredentials& creds1 = authenticationSvc.credentials(dbSet->replica( i ).connectionString(),"calib_writer");
             std::cout <<" Found ["<<creds1.numberOfItems()<<"] credential items"<<std::endl;
             for(int it=0;it<creds1.numberOfItems();++it){
               std::cout <<"For item="<<creds1.itemName(it)<<" value="<<creds1.valueForItem(creds1.itemName(it))<<std::endl;
             }
           } catch (coral::Exception& e){
             std::cout << "Exception: "<<e.what()<<std::endl;
           }
         }
         
       }
       
      delete dbSet;

      
    }
    }
    
    else {
      throw std::runtime_error("Could not retrieve an lookup service.");
    }
    std::cout << "Exiting..." << std::endl;
  }
  catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
    return 1;
  }
  std::cout << "[OVAL] Success" << std::endl;
  return 0;
}
