// $Id: DataEditor.cpp,v 1.5 2007/04/27 15:46:43 rado Exp $
#include "DataEditor.h"
#include "Statement.h"
#include "ISessionProperties.h"
#include "DomainProperties.h"
#include "ExpressionParser.h"

#include "OperationWithQuery.h"
#include "BulkOperation.h"
#include "BulkOperationWithQuery.h"

#include "RelationalAccess/ITableDescription.h"
#include "RelationalAccess/IColumn.h"
#include "RelationalAccess/SchemaException.h"

#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeSpecification.h"

#include "SealKernel/Service.h"
#include "SealKernel/MessageStream.h"

#include <sstream>

coral::MySQLAccess::DataEditor::DataEditor( const coral::MySQLAccess::ISessionProperties& properties, const coral::ITableDescription& description )
  : m_properties( properties )
  , m_tableDescription( description )
{
}

void coral::MySQLAccess::DataEditor::rowBuffer( coral::AttributeList& buffer )
{
  int numberOfColumns = m_tableDescription.numberOfColumns();
  for ( int iColumn = 0; iColumn < numberOfColumns; ++iColumn ) {
    const coral::IColumn& column = m_tableDescription.columnDescription( iColumn );
    buffer.extend( column.name(), column.type() );
  }
}

void coral::MySQLAccess::DataEditor::insertRow( const coral::AttributeList& dataBuffer )
{
  seal::MessageStream log( m_properties.domainProperties().service(), m_properties.domainProperties().service()->name(), seal::Msg::Debug );
  
  std::ostringstream os;

  os << "INSERT INTO " << m_properties.schemaName() << ".\"" << m_tableDescription.name() << "\" ( ";

  for ( coral::AttributeList::const_iterator iAttribute = dataBuffer.begin(); iAttribute != dataBuffer.end(); ++iAttribute )
  {
    if ( iAttribute != dataBuffer.begin() ) os << ", ";
    os << "\"" << iAttribute->specification().name() << "\"";
  }
  
  os << " ) VALUES ( ";

  for ( coral::AttributeList::iterator iAttribute = dataBuffer.begin(); iAttribute != dataBuffer.end(); ++iAttribute )
  {
    if ( iAttribute != dataBuffer.begin() )
      os << ", ";
    if( iAttribute->isNull() )
    {
      os << "NULL";
    }
    else
    {
      os << ":\"" << iAttribute->specification().name() << "\"";
      iAttribute->setNull( false );
    }
  }

  os << " )";

  std::string sql = os.str();
  
  coral::MySQLAccess::Statement statement( m_properties, sql );
  
  if ( ! statement.execute( dataBuffer ) ) // FIXME - Should be executed inside try/catch instead
    throw coral::DataEditorException( m_properties.domainProperties().service()->name(), "Could not insert a new row in the table", "ITableDataEditor::insertRow" );
}

coral::IOperationWithQuery* coral::MySQLAccess::DataEditor::insertWithQuery()
{
  std::ostringstream os;
  os << "INSERT INTO " << m_properties.schemaName() << ".\"" << m_tableDescription.name() << "\"";
  return new coral::MySQLAccess::OperationWithQuery( os.str(), m_properties );
}

coral::IBulkOperation* coral::MySQLAccess::DataEditor::bulkInsert( const coral::AttributeList& dataBuffer, int rowCacheSize )
{
  std::ostringstream os;
  os << "INSERT INTO " << m_properties.schemaName() << ".\"" << m_tableDescription.name() << "\" ( ";

  for ( coral::AttributeList::const_iterator iAttribute = dataBuffer.begin(); iAttribute != dataBuffer.end(); ++iAttribute )
  {
    if ( iAttribute != dataBuffer.begin() ) os << ", ";
    os << "\"" << iAttribute->specification().name() << "\"";
  }

  os << " ) VALUES ( ";

  for ( coral::AttributeList::const_iterator iAttribute = dataBuffer.begin(); iAttribute != dataBuffer.end(); ++iAttribute )
  {
    if ( iAttribute != dataBuffer.begin() ) os << ", ";
    os << ":\"" << iAttribute->specification().name() << "\"";
  }

  os << " )";

  return new coral::MySQLAccess::BulkOperation( m_properties, dataBuffer, rowCacheSize, os.str() );
}

coral::IBulkOperationWithQuery* coral::MySQLAccess::DataEditor::bulkInsertWithQuery( int dataCacheSize )
{
  std::ostringstream os;
  os << "INSERT INTO " << m_properties.schemaName() << ".\"" << m_tableDescription.name() << "\"";
  return new coral::MySQLAccess::BulkOperationWithQuery( m_properties, dataCacheSize, os.str() );
}

long coral::MySQLAccess::DataEditor::updateRows( const std::string& setClause, const std::string& condition, const coral::AttributeList& inputData )
{
  coral::MySQLAccess::ExpressionParser parser;
  parser.addToTableList( m_tableDescription );
  parser.appendToBindVariables( inputData );
  
  std::ostringstream os;

  os << "UPDATE " << m_properties.schemaName() << ".\"" << m_tableDescription.name() << "\" SET " << parser.parseExpression( setClause );
  
  if ( ! condition.empty() )
  {
    os << " WHERE " << parser.parseExpression( condition );
  }

  std::string sql = os.str();
  coral::MySQLAccess::Statement statement( m_properties, sql );

  if ( ! statement.execute( inputData ) ) // FIXME - Should be executed inside try/catch instead
    throw coral::DataEditorException( m_properties.domainProperties().service()->name(), "Could not updates rows in the table", "ITableDataEditor::updateRows" );
  
  return statement.numberOfRowsProcessed();
}

coral::IBulkOperation* coral::MySQLAccess::DataEditor::bulkUpdateRows( const std::string& setClause, const std::string& condition, const coral::AttributeList& inputData, int dataCacheSize )
{
  coral::MySQLAccess::ExpressionParser parser;
  parser.addToTableList( m_tableDescription );
  parser.appendToBindVariables( inputData );
  
  std::ostringstream os;
  os << "UPDATE " << m_properties.schemaName() << ".\"" << m_tableDescription.name() << "\" SET " << parser.parseExpression( setClause );
  
  if ( ! condition.empty() )
  {
    os << " WHERE " << parser.parseExpression( condition );
  }

  return new coral::MySQLAccess::BulkOperation( m_properties, inputData, dataCacheSize, os.str() );
}

long coral::MySQLAccess::DataEditor::deleteRows( const std::string& condition, const coral::AttributeList& conditionData )
{
  std::ostringstream os;
  os << "DELETE FROM \"" << m_properties.schemaName() << "\".\"" << m_tableDescription.name() << "\" ";
  
  if ( ! condition.empty() )
  {
    coral::MySQLAccess::ExpressionParser parser;
    parser.addToTableList( m_tableDescription );
    parser.appendToBindVariables( conditionData );
    
    os << " WHERE " << parser.parseExpression( condition );
  }
  
  std::string sql = os.str();
  coral::MySQLAccess::Statement statement( m_properties, sql );

  if ( ! statement.execute( conditionData ) ) // FIXME - Should be executed inside try/catch instead
    throw coral::DataEditorException( m_properties.domainProperties().service()->name(), "Could not delete rows from the table", "ITableDataEditor::updateRows" );

  return statement.numberOfRowsProcessed();
}

coral::IBulkOperation* coral::MySQLAccess::DataEditor::bulkDeleteRows( const std::string& condition, const coral::AttributeList& conditionData, int dataCacheSize )
{
  std::ostringstream os;
  os << "DELETE FROM \"" << m_properties.schemaName() << "\".\"" << m_tableDescription.name() << "\" ";
  
  if ( ! condition.empty() )
  {
    coral::MySQLAccess::ExpressionParser parser;
    parser.addToTableList( m_tableDescription );
    parser.appendToBindVariables( conditionData );

    os << " WHERE " << parser.parseExpression( condition );
  }

  return new coral::MySQLAccess::BulkOperation( m_properties, conditionData, dataCacheSize, os.str() );
}

