#include "CoralCommon/URIParser.h"

#include "SealKernel/ComponentLoader.h"
#include "SealKernel/MessageStream.h"

#ifdef WIN32
#define NOGDI
#define __LCC__
#include <winsock.h>
#endif
#include "mysql.h"

#include "Domain.h"
#include "DomainProperties.h"

#include "Connection.h"


DEFINE_SEAL_COMPONENT ( coral::MySQLAccess::Domain, "CORAL/RelationalPlugins/mysql" )

  coral::MySQLAccess::Domain::Domain( seal::Context* c )
    : seal::Service( c, classContextKey() ),
      m_flavorName( "MySQL" ),
      m_implementationName( "MySQL C API" ),
      m_implementationVersion( "" ),
      m_properties( new coral::MySQLAccess::DomainProperties( this ) ),
      m_initialized( false )
{
  // Get the client library version
  m_implementationVersion = mysql_get_client_info();
  // Perform domain specific setup
  this->initializeDomain();
}

coral::MySQLAccess::Domain::Domain( seal::Context* c, const std::string& key )
  : seal::Service( c, key ),
    m_flavorName( "MySQL" ),
    m_implementationName( "MySQL C API" ),
    m_implementationVersion( "" ),
    m_properties( new coral::MySQLAccess::DomainProperties( this ) ),
    m_initialized( false )
{
  // Get the client library version
  m_implementationVersion = mysql_get_client_info();
  this->initializeDomain();
}

coral::MySQLAccess::Domain::~Domain()
{
  delete m_properties;
}

std::string coral::MySQLAccess::Domain::flavorName() const
{
  return m_flavorName;
}

std::string coral::MySQLAccess::Domain::implementationName() const
{
  return m_implementationName;
}
      
std::string coral::MySQLAccess::Domain::implementationVersion() const
{
  return m_implementationVersion;
}

coral::IConnection* coral::MySQLAccess::Domain::newConnection( const std::string& uriString ) const
{
  return new coral::MySQLAccess::Connection( *m_properties, uriString );
}

std::pair<std::string, std::string> coral::MySQLAccess::Domain::decodeUserConnectionString(const std::string& cs ) const
{
  // Processing the connection string
  coral::URIParser parser;
  parser.setURI( cs );
  std::ostringstream osDatabase;
  osDatabase << parser.hostName();
  int portNumber = parser.portNumber();
  if ( portNumber != 0 ) osDatabase << ":" << portNumber;

  std::string tempSchema = parser.databaseOrSchemaName();
  std::string::size_type slashPosition = tempSchema.find( "/" );
  if ( slashPosition != std::string::npos ) {
    osDatabase << "/" << tempSchema.substr( 0, slashPosition );
    tempSchema = tempSchema.substr( slashPosition + 1 );
  }

  std::string schemaName = tempSchema;

  // In MySQL we're case sensitive except Windows based servers
//   for ( std::string::size_type i = 0; i < tempSchema.size(); ++i ) {
//     schemaName += std::toupper( tempSchema[i], std::locale::classic() );
//   }

  return std::make_pair( std::string( osDatabase.str() ), schemaName );
}

void coral::MySQLAccess::Domain::initializeDomain()
{  
  if( ! this->m_initialized )
  {
    seal::MessageStream log( this, this->name(), seal::Msg::Debug );

    seal::Handle< seal::ComponentLoader > loader = localContext()->component< seal::ComponentLoader >();

    if( loader )
    {
      log << seal::Msg::Debug << "MySQLAccess domain is loading its components" << seal::flush;
      
      // Load MySQLAccess specific components
      loader->loadAll( localContext(), "CORAL/mysql/" );
    }
  }

  this->m_initialized = true;
}
