// -*- C++ -*-
// $Id: Domain.h,v 1.5 2006/12/06 20:16:29 rado Exp $
#ifndef MYSQLACCESS_MYSQLDOMAIN_H
#define MYSQLACCESS_MYSQLDOMAIN_H 1

#include "RelationalAccess/IRelationalDomain.h"
#include "SealKernel/Service.h"

#include <memory>

namespace coral
{
  class IConnection;
  
  namespace MySQLAccess
  {

    class DomainProperties;

    /**
       @class Domain Domain.h
   
       Implementation of the IRelationalDomain interface for the MySQLAccess module
    */

    class Domain : public seal::Service, public virtual coral::IRelationalDomain
    {  
        DECLARE_SEAL_COMPONENT;

      public:
        /// Standard Constructor
        Domain( seal::Context* c );

        /// Standard Constructor with a component key
        Domain( seal::Context* c, const std::string& key );

        /// Standard Destructor
        virtual ~Domain();

        /// Returns the name of the RDBMS technology
        std::string flavorName() const;
      
        /// Returns the name of the software implementation library
        std::string implementationName() const;
      
        /// Returns the version of the software implementation library
        std::string implementationVersion() const;

        /// Creates a new session object. The user acquires ownership of this object.
        coral::IConnection* newConnection( const std::string& uriString ) const;

        std::pair<std::string, std::string> decodeUserConnectionString(const std::string&) const;

        bool isAuthenticationRequired(const std::string&) const
        {
          return true;
        }
        
      private:
        /// Perform service internal setup
        void initializeDomain();

      private:
        /// The flavor name
        std::string                           m_flavorName;
        /// The name of the software implementation library
        std::string                           m_implementationName;
        /// The version of the software implementation library
        std::string                           m_implementationVersion;
        /// The domain properties
        coral::MySQLAccess::DomainProperties* m_properties;
        /// Is it initialized?
        bool                                  m_initialized;
    };
  }
}

#endif // MYSQLACCESS_MYSQLDOMAIN_H
