// $Id: ExpressionParser.cpp,v 1.5 2006/09/25 10:37:16 rado Exp $
#include "ExpressionParser.h"

#include"RelationalAccess/ITableDescription.h"
#include "RelationalAccess/IColumn.h"

#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeSpecification.h"

#include <sstream>

coral::MySQLAccess::ExpressionParser::ExpressionParser()
  : m_tablesAndColumns()
  , m_tableAliases()
  , m_bindVariables()
{
}

coral::MySQLAccess::ExpressionParser::~ExpressionParser()
{
}

void coral::MySQLAccess::ExpressionParser::addToTableList( const coral::ITableDescription& description )
{
  std::set< std::string >& variables = m_tablesAndColumns[ description.name() ];
  int numberOfColumns = description.numberOfColumns();
  for ( int i = 0; i < numberOfColumns; ++i )
  {
    variables.insert( description.columnDescription( i ).name() );
  }
}

void coral::MySQLAccess::ExpressionParser::addToTableList( const std::string& table, const std::vector<std::string>& outputVariables )
{
  std::set< std::string >& variables = m_tablesAndColumns[ table ];
  for ( std::vector<std::string>::const_iterator iVariable = outputVariables.begin(); iVariable != outputVariables.end(); ++iVariable )
    variables.insert( *iVariable );
}

void coral::MySQLAccess::ExpressionParser::addToAliases( const std::string& alias )
{
  if( m_tableAliases.find( alias ) == m_tableAliases.end() )
    m_tableAliases.insert( alias );
}

void coral::MySQLAccess::ExpressionParser::appendToBindVariables( const coral::AttributeList& bindData )
{
  for ( coral::AttributeList::const_iterator iBindVariable = bindData.begin(); iBindVariable != bindData.end(); ++iBindVariable )
  {
    m_bindVariables.insert( iBindVariable->specification().name() );
  }
}

std::string coral::MySQLAccess::ExpressionParser::parseExpression( const std::string& expression ) const
{
  std::ostringstream os;
  std::string lastWord = "";
  bool withinQuotes = false;
  std::string::size_type length = expression.size();
  char previous = '\0';

  for ( std::string::size_type iCharacter = 0; iCharacter < length; ++iCharacter )
  {
    char c = expression[iCharacter];

    if ( withinQuotes )
    {
      if ( c == '\'' ) withinQuotes = false;
      os << c;
      continue;
    }

    if ( c == '\'' )
    {
      os << this->decorateWord( lastWord, previous );
      lastWord = "";
      os << c;
      withinQuotes = true;
      continue;
    }

    if ( c == ' ' || c == '\t' || c == '\n' || c == '\r' ||
         c == '`' || c == '~' || c == '!' || c == '@' || c == '%' ||
         c == '^' || c == '&' || c == '*' || c == '(' || c == ')' ||
         c == '-' || c == '+' || c == '=' || c == '[' || c == ']' ||
         c == '{' || c == '}' || c == ';' || c == ':' || c == '"' ||
         c == '\\' || c == '|' || c == '<' || c == '>' || c == ',' ||
         c == '.' || c == '?' || c == '/' )
    {
      std::string finalWord = this->decorateWord( lastWord, previous );
      os << finalWord;
      lastWord = "";
      os << c;

      if ( finalWord.size() > 0 && finalWord[0] == '"' && previous == ':' )
        previous = '\0';
      else if ( c != ' ' && c != '\t' && c != '\n' && c != '\r' )
        previous = c;
    }
    else
    {
      lastWord += c;
    }
  }

  os << this->decorateWord( lastWord, previous );

  return os.str();
}

std::string coral::MySQLAccess::ExpressionParser::decorateWord( const std::string& expression, char previous ) const
{
  if ( expression.empty() )
    return "";

  if ( previous == ':' )
  { // This is a bind variable
    if ( m_bindVariables.find( expression ) != m_bindVariables.end() )
      return "\"" + expression + "\"";
    else
      return expression;
  }
  else if ( previous == '.' )
  { // This is a variable in a table
    for ( std::map< std::string, std::set< std::string > >::const_iterator iTable = m_tablesAndColumns.begin(); iTable != m_tablesAndColumns.end(); ++iTable )
    {
      if ( iTable->second.find( expression ) != iTable->second.end() )
        return "\"" + expression + "\"";
    }
    return expression;
  }
  else
  {
    if ( m_tablesAndColumns.find( expression ) !=  m_tablesAndColumns.end() || m_tableAliases.find( expression ) != m_tableAliases.end() )
    {
      // table name or a table alias
      return "\"" + expression + "\"";
    }
    else if ( m_tablesAndColumns.size() > 0 )
    { // This is a variable in the table
      std::map< std::string, std::set< std::string > >::const_iterator iTable = m_tablesAndColumns.begin();
      if ( iTable->second.find( expression ) != iTable->second.end() )
        return "\"" + expression + "\"";
    }
    return expression;
  }
}
