#include "SessionProperties.h"
#include "DomainProperties.h"
#include "TypeConverter.h"

#include "RelationalAccess/ISession.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/SessionException.h"

#include "SealKernel/Service.h"

coral::MySQLAccess::SessionProperties::SessionProperties( const DomainProperties& domainProperties,
                                                          const std::string& connectionString, const std::string& schemaName,
                                                          MYSQL*& mysql, bool& connected, boost::mutex& conlock,
                                                          coral::ISession& session )
  : m_domainProperties( domainProperties )
  , m_connectionString( connectionString )
  , m_session( session )
  , m_accessMode( coral::Update )
  , m_connectionHandle( mysql )
  , m_isConnected( connected )
  , m_userSessionStarted( false )
  , m_monitoringService( 0 )
  , m_typeConverter( 0 )
  , m_schemaName( schemaName )
  , m_serverVersion()
  , m_serverRevision()
  , m_lock( conlock )
{
  // Make sure MySQL type converter is available
  m_typeConverter = m_domainProperties.service()->localContext()->component<coral::MySQLAccess::TypeConverter>();
  
  if( !m_typeConverter )
  //  throw coral::TypeConverterException( "MySQLAccess", "Type converter not loaded", "MySQLAccess::SessionProperties::typeConverter" );
    m_typeConverter = new coral::MySQLAccess::TypeConverter( m_domainProperties.service()->localContext()  );
}

coral::MySQLAccess::SessionProperties::~SessionProperties()
{
}

bool coral::MySQLAccess::SessionProperties::isTransactionActive() const
{
  return m_session.transaction().isActive();
}

coral::ITypeConverter& coral::MySQLAccess::SessionProperties::typeConverter()
{
  return( *(m_typeConverter.get()) );
}

const coral::ITypeConverter& coral::MySQLAccess::SessionProperties::typeConverter() const
{
  return( *m_typeConverter );
}

coral::ISchema& coral::MySQLAccess::SessionProperties::schema() const
{
  return m_session.nominalSchema();
}

std::string coral::MySQLAccess::SessionProperties::schemaName() const
{
  return m_schemaName;
}

std::string coral::MySQLAccess::SessionProperties::serverVersion() const
{
  return m_serverVersion;
}

void coral::MySQLAccess::SessionProperties::setServerVersion( const std::string& serverVersion )
{
  m_serverVersion = serverVersion;
}

coral::MySQLAccess::ServerRevision coral::MySQLAccess::SessionProperties::serverRevision() const
{
  return m_serverRevision;
}

void coral::MySQLAccess::SessionProperties::setServerRevision( unsigned long major, unsigned long minor, unsigned long revision )
{
  m_serverRevision.major    = major;
  m_serverRevision.minor    = minor;
  m_serverRevision.revision = revision;  
}
