#include "TestBase.h"

#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/RelationalServiceException.h"

#include "PluginManager/PluginManager.h"
#include "SealKernel/Context.h"
#include "SealKernel/ComponentLoader.h"

#include <iostream>

TestBase::TestBase( seal::Context* ctx )
  : m_context( 0 )
{
  if( ctx == 0 )
    this->m_context = new seal::Context;
  else
    m_context = ctx;
  
  seal::PluginManager* pm = seal::PluginManager::get();
  pm->initialise();
  seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( m_context.get() );
  loader->load( "CORAL/RelationalPlugins/mysql" );
  const char* authvar = getenv( "CORAL_AUTH_PATH" );

  if( authvar ) 
  {    
    loader->load( "CORAL/Services/XMLAuthenticationService" );
  }
}

TestBase::~TestBase()
{
}

coral::ISession* TestBase::connect( const std::string& connectionString, const std::string& userName, const std::string& password, coral::AccessMode mode )
{
  seal::IHandle<coral::IRelationalDomain> domain = m_context->query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/mysql" );
  if ( ! domain )
  {
    throw coral::NonExistingDomainException( "mysql" );
  }

  std::cout << "Retrieved relational domain \"" << domain->flavorName()
            << "\" implementation \"" << domain->implementationName()
            << "\" version " << domain->implementationVersion() << std::endl;

  std::pair<std::string,std::string> cs = domain->decodeUserConnectionString( connectionString );

  std::cout << "Connecting to: " << cs.first << " ing schema: " << cs.second << std::endl;
  
  coral::IConnection* connection = domain->newConnection( cs.first );

  // FIXME !!! call in try..catch..block
  coral::ISession* session = connection->newSession( cs.second /* the scheman name */, mode );

  if ( session ) {
    session->startUserSession( userName, password );
  }

  return session;
}
