#include "MultipleSessionsApp.h"
#include "../Common/Reporter.h"
#include "ThreadBody.h"
#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/RelationalServiceException.h"
#include "PluginManager/PluginManager.h"
#include "SealKernel/Context.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/IMessageService.h"
#include <boost/thread.hpp>
#include <vector>
#include <memory>


MultipleSessionsApp::MultipleSessionsApp( const std::string& connectionString,
                                          const std::string& userName,
                                          const std::string& password,
                                          Reporter& reporter ):
  m_context( new seal::Context ),
  m_connectionString( connectionString ),
  m_userName( userName ),
  m_password( password ),
  m_reporter( reporter )
{
  seal::PluginManager* pm = seal::PluginManager::get();
  pm->initialise();
  seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( m_context.get() );

  if ( ! loader->load( "SEAL/Services/MessageService" ) ) {
    throw std::runtime_error( "Could not load the seal message service" );
  }

  if ( ! loader->load( "CORAL/RelationalPlugins/oracle" ) ) {
    throw std::runtime_error( "Could not load the OracleAccess plugin" );
  }

}


MultipleSessionsApp::~MultipleSessionsApp()
{}

void
MultipleSessionsApp::run()
{
  seal::IHandle<coral::IRelationalDomain> iHandle = m_context->query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/oracle" );
  if ( ! iHandle ) {
    throw coral::NonExistingDomainException( "oracle" );
  }

  std::pair<std::string, std::string> connectionAndSchema = iHandle->decodeUserConnectionString( m_connectionString );

  std::auto_ptr<coral::IConnection> connection( iHandle->newConnection( connectionAndSchema.first ) );
  connection->connect();

  // Launch the parallel threads here that share the same connection...
  std::vector< ThreadBody* > threadBodies;

  for ( int i = 0; i < 10; ++i )
    threadBodies.push_back( new ThreadBody( *connection, i, m_reporter,
                                            connectionAndSchema.second,
                                            m_userName, m_password ) );

  std::vector< boost::thread* > threads;
  for ( int i = 0; i < 10; ++i )
    threads.push_back( new boost::thread( *( threadBodies[i] ) ) );

  m_reporter.reportToSTDOUT( "Waiting for the threads..." );

  for ( int i = 0; i < 10; ++i )
    threads[i]->join();

  for ( int i = 0; i < 10; ++i ) {
    delete threads[i];
    delete threadBodies[i];
  }

}
