#ifndef RELATIONALACCESS_CONNECTIONSERVICE_H
#define RELATIONALACCESS_CONNECTIONSERVICE_H

#include "IConnectionService.h"
#include "SealKernel/MessageStream.h"

namespace seal {
  class Context;
}

namespace coral {

  /**
   * Class ConnectionService
   * 
   * Proxy implementation for the IConnectionService interface, hidding all the complexities of the SEAL component model
   */
  class ConnectionService : virtual public IConnectionService {
  public:
    /// Constructor initialized with a context. If not specified a new context is created internally
    explicit ConnectionService( seal::Context* parentContext = 0 );

    /// Destructor
    ~ConnectionService();

    /**
     * Returns a session proxy object for the specified connection string
     * and access mode.
     */
    ISessionProxy* connect( const std::string& connectionName,
                            AccessMode accessMode = Update );

    /**
     * Returns a session proxy object for the specified connection string, role
     * and access mode.
     */
    ISessionProxy* connect( const std::string& connectionName,
                            const std::string& asRole,
                            AccessMode accessMode = Update );
    /**
     * Returns the configuration object for the service.
     */
    IConnectionServiceConfiguration& configuration();

    /**
     * Cleans up the connection pool from the unused connection, according to
     * the policy defined in the configuration.
     */
    void purgeConnectionPool();

    /**
     * Returns the monitoring reporter
     */
    const IMonitoringReporter& monitoringReporter() const;    

    /**
     * Returns the object which controls the web cache
     */
    IWebCacheControl& webCacheControl();

    // Set the user defined verbosity level
    void setMessageVerbosityLevel( seal::Msg::Level level );

  private:
    /// No copy constructor or assignment operator
    ConnectionService( const ConnectionService& );
    ConnectionService& operator=( const ConnectionService& );

  private:
    seal::Context*       m_context;
    IConnectionService*  m_connectionService;
  };

}






// INLINE METHODS



inline coral::ISessionProxy*
coral::ConnectionService::connect( const std::string& connectionName,
                                   coral::AccessMode accessMode )
{
  return m_connectionService->connect( connectionName, accessMode );
}


inline coral::ISessionProxy*
coral::ConnectionService::connect( const std::string& connectionName,
                                   const std::string& asRole,
                                   coral::AccessMode accessMode )
{
  return m_connectionService->connect( connectionName, asRole, accessMode );
}


inline coral::IConnectionServiceConfiguration&
coral::ConnectionService::configuration()
{
  return m_connectionService->configuration();
}


inline void
coral::ConnectionService::purgeConnectionPool()
{
  m_connectionService->purgeConnectionPool();
}


inline const coral::IMonitoringReporter&
coral::ConnectionService::monitoringReporter() const
{
  return m_connectionService->monitoringReporter();
}


inline coral::IWebCacheControl&
coral::ConnectionService::webCacheControl()
{
  return m_connectionService->webCacheControl();
}


#endif
