#ifndef RELATIONALACCESS_ISESSIONPROXY_H
#define RELATIONALACCESS_ISESSIONPROXY_H

#include <string>

namespace coral {

  // forward declarations
  class ISchema;
  class ITransaction;
  class ITypeConverter;
  class ISessionProperties;

  /**
   * Class ISessionProxy
   * 
   * Interface for a proxy class to an ISession object.
   * Whenever the user retrieves the schema, transaction or
   * typeconverter objects, the connection is physically probed
   * and if necessary (and being outside of an update transaction)
   * a new connection is established.
   * In case the connection has been broken in the middle of an
   * update transaction, an exception is raised.
   */
  class ISessionProxy
  {
  public:
    /**
     * Virtual destructor
     */
    virtual ~ISessionProxy() {} 
  
    /**
     * Returns the session info object.
     * Since the underlying connection is probed before the object info construction, it always contains vaild data.
     */
    virtual ISessionProperties& properties() = 0;    

    /**
     * Returns the working schema of the connection.
     */
    virtual ISchema& nominalSchema() = 0;    
  
    /**
     * Returns a reference to the ISchema object corresponding to the specified name.
     */
    virtual ISchema& schema( const std::string& schemaName ) = 0;

    /**
     * Returns the transaction handle of the active session.
     */
    virtual ITransaction& transaction() = 0;    
  
    /**
     * Returns the type converter of the active session
     */
    virtual ITypeConverter& typeConverter() = 0;

  };

}

#endif
