#include "RelationalAccess/ConnectionService.h"
#include "CoralBase/Exception.h"
#include "SealKernel/Context.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/IMessageService.h"
#include "PluginManager/PluginManager.h"

coral::ConnectionService::ConnectionService( seal::Context* parentContext ):
  m_context( ( parentContext ) ? parentContext : new seal::Context ),
  m_connectionService( 0 )
{
  // Increment the reference counter of the context
  intrusive_ptr_add_ref( m_context );

  // Initialize the plugin manager
  seal::PluginManager::get()->initialise();

  // Load if needed and if found the message service
  std::vector< seal::Handle<seal::IMessageService> > v_msvc;
  m_context->root()->query( v_msvc );
  if ( v_msvc.empty() ) {
    seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( m_context->root() );
    loader->load( "SEAL/Services/MessageService" );
  }

  // Load the connection service
  std::vector< seal::IHandle<coral::IConnectionService> > v_svc;
  m_context->query( v_svc );
  bool ok = true;
  if ( v_svc.empty() ) {
    ok = false;
    seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( m_context );
    loader->load( "CORAL/Services/ConnectionService" );
    m_context->query( v_svc );
    if ( v_svc.empty() ) {
      throw coral::Exception( "Could not load the CORAL ConnectionService",
                              "ConnectionService::ConnectionService",
                              "RelationalAccess" );
    }
  }
  m_connectionService = v_svc.front().get();
}


coral::ConnectionService::~ConnectionService()
{
  if ( m_context ) {
    intrusive_ptr_release( m_context );
  }
}

void
coral::ConnectionService::setMessageVerbosityLevel( seal::Msg::Level level )
{
  std::vector< seal::Handle<seal::IMessageService> > v_msvc;
  m_context->root()->query( v_msvc );
  if( !v_msvc.empty() ){
    seal::Handle<seal::IMessageService>& msgSvc = v_msvc.front();
    msgSvc->setOutputLevel( level );
  }
}

