#include <iostream>
#include <stdexcept>
#include "PluginManager/PluginManager.h"
#include "SealKernel/ComponentLoader.h"
#include "CoralBase/Exception.h"
#include "SealKernel/Context.h"
#include "RelationalAccess/IRelationalService.h"

int main( int, char** )
{
  try {
    seal::PluginManager* pm = seal::PluginManager::get();
    pm->initialise();

    seal::Handle< seal::Context > context = new seal::Context();
    seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( context.get() );

    loader->load( "CORAL/Services/RelationalService" );

    std::vector< seal::IHandle<coral::IRelationalService> > v_rSvc;
    context->query( v_rSvc );
    if ( v_rSvc.empty() ) {
      throw std::runtime_error( "Could not locate the relational service" );
    }

    const seal::IHandle<coral::IRelationalService>& iSvc = v_rSvc.front();

    std::vector< std::string > technologies = iSvc->availableTechnologies();
    std::cout << "Technologies : " << std::endl;
    for ( std::vector< std::string >::const_iterator iTechnology = technologies.begin();
          iTechnology != technologies.end(); ++iTechnology )
      std::cout << "     " << *iTechnology << std::endl;

  }
  catch ( coral::Exception& e ) {
    std::cerr << "CORAL exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( std::exception& e ) {
    std::cerr << "C++ exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Funny error ..." << std::endl;
    return 1;
  }
  std::cout << "[OVAL] Success" << std::endl;
  return 0;
}
