#ifndef SQLITE_CONNECTIONPROPERTIES_H
#define SQLITE_CONNECTIONPROPERTIES_H
#include "SealBase/Filename.h"
#include <string>
namespace coral {

  class ITypeConverter;

  namespace SQLiteAccess {

    class TypeConverter;
    class DomainProperties;

    /**
     * Class ConnectionProperties
     *
     * A class holding the running parameters of a database connection
     */

    class ConnectionProperties
    {
    public:
      /// Constructor
      ConnectionProperties( const DomainProperties& domainProperties,
                            const std::string& connectionString );

      /// Destructor
      virtual ~ConnectionProperties();

      /// Sets the read-only flag
      void setReadOnly( bool flag );

      /// Sets the database file name
      void setFileName( const seal::Filename& m_inFileName );

      /// Returns the database file name
      seal::Filename databaseFileName() const;

      /// Returns the domain properties
      const DomainProperties& domainProperties() const;

      /// Returns the connection string
      std::string connectionString() const;

      /// Returns the type converter
      coral::ITypeConverter& typeConverter();
      const coral::ITypeConverter& typeConverter() const;

      /// Returns the readOnly flag
      bool isReadOnly() const;

    private:
      /// The domain properties
      const DomainProperties& m_domainProperties;

      /// The connection string
      std::string m_connectionString;

      /// The type converter
      TypeConverter* m_typeConverter;

      /// The read-only flag
      bool m_isReadOnly;

      /// The sqlite file name
      seal::Filename m_inFileName;
      /// The server version
      //std::string           m_serverVersion;
    };

  }

}


// Inline methods

inline const coral::SQLiteAccess::DomainProperties&
coral::SQLiteAccess::ConnectionProperties::domainProperties() const
{
  return m_domainProperties;
}


inline std::string
coral::SQLiteAccess::ConnectionProperties::connectionString() const
{
  return m_connectionString;
}

inline bool
coral::SQLiteAccess::ConnectionProperties::isReadOnly() const
{
  return m_isReadOnly;
}

inline void
coral::SQLiteAccess::ConnectionProperties::setReadOnly( bool flag )
{
  m_isReadOnly = flag;
}

inline void 
coral::SQLiteAccess::ConnectionProperties::setFileName( const seal::Filename& inFileName ){
  m_inFileName=inFileName;
}
/// Returns the database file name
inline seal::Filename 
coral::SQLiteAccess::ConnectionProperties::databaseFileName() const{
  return m_inFileName;
}
#endif
