#include <iostream>
#include <stdexcept>
#include <memory>
#include "RelationalAccess/RelationalServiceException.h"
#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "RelationalAccess/IBulkOperation.h"
#include "CoralBase/AttributeList.h"
#include "SealKernel/Context.h"
#include "SealBase/IntBits.h"
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ICursor.h"
#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/Exception.h"
#include "PluginManager/PluginManager.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/MessageStream.h"
#include <sstream>
#include <climits>
#include <float.h>
int main( int, char** )
{
  std::string tablename="bulktest";
  std::string con="sqlite_file:zhen.db";
  seal::Handle< seal::Context > context=new seal::Context;
  seal::PluginManager* pm = seal::PluginManager::get();
  pm->initialise();
  try {
    seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader(context.get());
    loader->load( "SEAL/Services/MessageService" );
    std::vector< seal::Handle<seal::IMessageService> > v_msgSvc;
    context->query( v_msgSvc );
    if ( ! v_msgSvc.empty() ) {
      seal::Handle<seal::IMessageService>& msgSvc = v_msgSvc.front();
      msgSvc->setOutputLevel( seal::Msg::Debug );
    }
    loader->load( "CORAL/RelationalPlugins/sqlite" );
    seal::IHandle<coral::IRelationalDomain> iHandle=context->query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/sqlite" );
    if ( ! iHandle ) {
      throw coral::NonExistingDomainException( "sqlite" );
    }
    std::pair<std::string, std::string> connectionAndSchema = iHandle->decodeUserConnectionString( con );
    coral::IConnection*  connection=iHandle->newConnection( connectionAndSchema.first );
    connection->connect();
    coral::ISession* session=connection->newSession(connectionAndSchema.second);  
    std::string userName, password;
    session->startUserSession( userName, password );
    //domainproperties=new coral::SQLiteAccess::DomainProperties(context->component<coral::SQLiteAccess::Domain>().get());
    //session=new coral::SQLiteAccess::Session(*domainproperties, con);
    //sessionproperties=new coral::SQLiteAccess::SessionProperties(*domainproperties, con,*session);
    /*if( session ){
      session->connect();
      session->startUserSession(userName,password) ;
      }
    */
    session->transaction().start(false) ;
    coral::ISchema& schema = session->nominalSchema();
    coral::TableDescription* desc=new coral::TableDescription;
    desc->setName( tablename );
    desc->insertColumn( "myfloat", coral::AttributeSpecification::typeNameForId(typeid(float)) );
    desc->insertColumn( "mydouble", coral::AttributeSpecification::typeNameForId(typeid(double)) );
    desc->insertColumn( "mylongdouble", coral::AttributeSpecification::typeNameForId(typeid(long double)) );
    desc->insertColumn( "myint", coral::AttributeSpecification::typeNameForId(typeid(int)) );
    desc->insertColumn( "myunsignedint", coral::AttributeSpecification::typeNameForId(typeid(unsigned int)) );
    desc->insertColumn( "myshort", coral::AttributeSpecification::typeNameForId(typeid(short)) );
    desc->insertColumn( "myunsignedshort", coral::AttributeSpecification::typeNameForId(typeid(unsigned short)) );
    desc->insertColumn( "mylong", coral::AttributeSpecification::typeNameForId(typeid(long)) );
    desc->insertColumn( "myunsignedlong", coral::AttributeSpecification::typeNameForId(typeid(unsigned long)) );
    desc->insertColumn( "mysleast", coral::AttributeSpecification::typeNameForId(typeid(seal::IntBits<64>::SLeast)) );
    desc->insertColumn( "myuleast", coral::AttributeSpecification::typeNameForId(typeid(seal::IntBits<64>::ULeast)) );
    desc->insertColumn( "mybool", coral::AttributeSpecification::typeNameForId(typeid(bool)) );
    desc->insertColumn( "mychar", coral::AttributeSpecification::typeNameForId(typeid(char)) );
    desc->insertColumn( "myuschar", coral::AttributeSpecification::typeNameForId(typeid(unsigned char)) );
    desc->insertColumn( "mystring", coral::AttributeSpecification::typeNameForId(typeid(std::string)) );
    desc->setNotNullConstraint( "mystring" );
    schema.dropIfExistsTable(tablename);
    std::cout<<1<<std::endl;
    schema.createTable(*desc);
    std::cout<<2<<std::endl;

    coral::ISchema& workingSchema = session->nominalSchema();
      std::cout<<"he"<< &workingSchema<<std::endl;
      coral::ITable& table=workingSchema.tableHandle(tablename);
      std::cout<<"got handle"<<std::endl;
      // Costructing a row buffer.
      coral::AttributeList data;
      std::cout<<"about to construct row buf"<<std::endl;
      table.dataEditor().rowBuffer(data);
      // Adding new rows
      std::cout << "Inserting new rows into the table." << std::endl;
      coral::IBulkOperation* rowInserter = table.dataEditor().bulkInsert(data,3);
      for ( int i = 0; i < 5; ++i ) {
	std::ostringstream os;
	os << i;
	std::string stri=os.str();
	///some harmless dummy values
	data["myfloat"].data<float>()= 37.623 ;
	data["mydouble"].data<double>()=2.2e-308 ;
	data["mylongdouble"].data<long double>()= -1897.23;
	data["myint"].data<int>()= -41928 ;//s32
	data["myunsignedint"].data<unsigned int>()=4294967295U;//u32
	data["myshort"].data<short>()=100 ;//s16
	data["myunsignedshort"].data<unsigned short>()=65535; //u16
	data["mylong"].data<long>()=814748647L;//s32
	data["myunsignedlong"].data<unsigned long>()= 4294967295U ; //u32
	data["mysleast"].data< seal::IntBits<64>::SLeast >()= 9223372036854775807LL;//s64
	data["myuleast"].data< seal::IntBits<64>::ULeast >()= 18446744073709551615ULL;//u64
	data["mybool"].data<bool>()= true ;
	data["mychar"].data<char>()= -1 ;
	data["myuschar"].data<unsigned char>()= 'c' ;
	data["mystring"].data<std::string>()=std::string("c")+stri ;
	///lower limits
	if(i==1){
	  data["myfloat"].data<float>()=FLT_MIN;
	  data["mydouble"].data<double>()=DBL_MIN;
	  data["mylongdouble"].data<long double>()=DBL_MIN;
	  data["myint"].data<int>()=INT_MIN;
	  data["myunsignedint"].data<unsigned int>()=0;
	  data["myshort"].data<short>()= SHRT_MIN ;//s16
	  data["myunsignedshort"].data<unsigned short>()=0;
	  data["mylong"].data<long>()=LONG_MIN;
	  data["myunsignedlong"].data<unsigned long>()=0;
	  data["mysleast"].data< seal::IntBits<64>::SLeast >()=LONG_LONG_MIN;
	  data["myuleast"].data< seal::IntBits<64>::ULeast >()=0;
	  data["mybool"].data<bool>()= false;
	  data["mychar"].data<char>()=CHAR_MIN;
	  data["myuschar"].data<unsigned char>()=0;
	}
	///upper limits
	if(i==2){
	  data["myfloat"].data<float>()=FLT_MAX;
	  data["mydouble"].data<double>()=DBL_MAX;
	  data["mylongdouble"].data<long double>()=DBL_MAX;
	  data["myint"].data<int>()=INT_MAX;
	  data["myunsignedint"].data<unsigned int>()=UINT_MAX;
	  data["myshort"].data<short>()= SHRT_MAX ;//s16
	  data["myunsignedshort"].data<unsigned short>()=USHRT_MAX;
	  data["mylong"].data<long>()=LONG_MAX;
	  data["myunsignedlong"].data<unsigned long>()=ULONG_MAX;
	  data["mysleast"].data< seal::IntBits<64>::SLeast >()=LONG_LONG_MAX;
	  data["myuleast"].data< seal::IntBits<64>::ULeast >()=ULONG_LONG_MAX;
	  data["mybool"].data<bool>()= false ;
	  data["mychar"].data<char>()=CHAR_MAX;
	  data["myuschar"].data<unsigned char>()=UCHAR_MAX;
	}
	rowInserter->processNextIteration();
      }
      rowInserter->flush();
      delete rowInserter;
      std::cout<<"querying"<<std::endl;
      coral::IQuery* query=table.newQuery();
      query->setRowCacheSize( 10 );
      coral::ICursor& cursor = query->execute();
      int counter=0;
      while( cursor.next() ) {
	std::cout<<"counter "<<counter<<std::endl;
	const coral::AttributeList& row = cursor.currentRow();
	if( counter==0 ){
	  float myf=row["myfloat"].data<float>();
	  std::cout<<myf<<std::endl;
	  if(myf!=37.623f){
	    std::cout<<"wrong myf "<<myf<<std::endl;
	  }
	  double mydl=row["mydouble"].data<double>();
	  if(mydl!=2.2e-308) {
	    std::cout<<"wrong mydl "<<mydl<<std::endl;
	  }
	  long double mylongdl=row["mylongdouble"].data<long double>();
	  if(mylongdl!=-1897.23){
	    std::cout<<"wrong mylongdl "<<mylongdl<<std::endl;
	  }
	  int myi=row["myint"].data<int>();
	  if(myi!=-41928){
	    std::cout<<"wrong myi "<<myi<<std::endl;
	  }
	  unsigned int myui=row["myunsignedint"].data<unsigned int>();
	  if(myui!=4294967295U){
	    std::cout<<"wrong myui "<<myui<<std::endl;
	  }
	  seal::IntBits<64>::SLeast mysll=row["mysleast"].data< seal::IntBits<64>::SLeast >();
	  if(mysll!=9223372036854775807LL){
	    std::cout<<"wrong mysll "<<mysll<<std::endl;
	  }
	  seal::IntBits<64>::ULeast myull=row["myuleast"].data< seal::IntBits<64>::ULeast >();
	  if(myull!=18446744073709551615ULL){
	    std::cout<<"wrong myull "<<myull<<std::endl;
	  }
	  bool mybool=row["mybool"].data<bool>();
	  if(mybool!=true){
	    std::cout<<"wrong mybool "<<mybool<<std::endl;
	  }
	  char mychar=row["mychar"].data<char>();
	  if(mychar!=-1){
	    std::cout<<"wrong mychar "<<mychar<<std::endl;
	  }
	  unsigned char myuchar=row["myuschar"].data<unsigned char>();
	  if(myuchar!='c'){
	    std::cout<<"wrong myuchar "<<myuchar<<std::endl;
	  }
	  std::cout<<"myuchar "<<myuchar<<std::endl;
	}
	if( counter==1 ){
	  float myf=row["myfloat"].data<float>();
	  std::cout<<"myf "<<myf<<std::endl;
	  if(myf!=FLT_MIN){
	    std::cout <<"wrong myf "<<myf<<std::endl;
	  }
	  double mydl=row["mydouble"].data<double>();
	  if(mydl!=DBL_MIN){
	    std::cout <<"wrong mydl "<<mydl<<std::endl;
	  }
	  long double mylongdl=row["mylongdouble"].data<long double>();
	  if(mylongdl!=DBL_MIN){
	    std::cout <<"wrong mylongdl "<<mylongdl<<std::endl;
	  }
	  int myi=row["myint"].data<int>();
	  if(myi!=INT_MIN){
	    std::cout <<"wrong myi "<<myi<<std::endl;
	  }
	  unsigned int myui=row["myunsignedint"].data<unsigned int>();
	  if(myui!=0){
	    std::cout <<"wrong myui "<<myui<<std::endl;
	  }
	  short myshort=row["myshort"].data<short>();
	  if(myshort!=SHRT_MIN){
	    std::cout <<"wrong myshort "<<myshort<<std::endl;
	  }
	  unsigned short ushort=row["myunsignedshort"].data<unsigned short>();
	  if(ushort!=0){
	    std::cout <<"wrong ushort "<<ushort<<std::endl;
	  }
	  long myl=row["mylong"].data<long>();
	  if(myl!=LONG_MIN){
	    std::cout <<"wrong myl "<<myl<<std::endl;
	  }
	  unsigned long myul=row["myunsignedlong"].data<unsigned long>();
	  if(myul!=0){
	    std::cout <<"wrong myul "<<myul<<std::endl;
	  }
	  seal::IntBits<64>::SLeast mysll=row["mysleast"].data< seal::IntBits<64>::SLeast >();
	  if(mysll!=LONG_LONG_MIN){
	    std::cout <<"wrong mysll "<<mysll<<std::endl;
	  }
	  seal::IntBits<64>::ULeast myull=row["myuleast"].data< seal::IntBits<64>::ULeast >();
	  if(myull!=0){
	    std::cout <<"wrong myull "<<myull<<std::endl;
	  }
	  bool mybool=row["mybool"].data<bool>();
	  if(mybool!=false){
	    std::cout <<"wrong mybool "<<mybool<<std::endl;
	  }
	  char mychar=row["mychar"].data<char>();
	  if(mychar!=CHAR_MIN){
	    std::cout <<"wrong mychar "<<mychar<<std::endl;
	  }
	  unsigned char myuchar=row["myuschar"].data<unsigned char>();
	  if(myuchar!=0){
	    std::cout <<"wrong myuchar "<<myuchar<<std::endl;
	  }
	}
	if( counter==2 ){
	  float myf=row["myfloat"].data<float>();
	  if(myf!=FLT_MAX){
	    std::cout<<"wrong myf "<<myf<<std::endl;
	  }
	  double mydl=row["mydouble"].data<double>();
	  if(mydl!=DBL_MAX){
	    std::cout<<"wrong mydl "<<mydl<<std::endl;
	  }
	  long double mylongdl=row["mylongdouble"].data<long double>();
	  if(mylongdl!=DBL_MAX){
	    std::cout<<"wrong mylongdl "<<mylongdl<<std::endl;
	  }
	  int myi=row["myint"].data<int>();
	  if(myi!=INT_MAX){
	    std::cout<<"wrong myi "<<myi<<std::endl;
	  }
	  unsigned int myui=row["myunsignedint"].data<unsigned int>();
	  if(myui!=UINT_MAX){
	    std::cout<<"wrong myui "<<myui<<std::endl;
	  }
	  short myshort=row["myshort"].data<short>();
	  if(myshort!=SHRT_MAX){
	    std::cout<<"wrong myshort "<<myshort<<std::endl;
	  }
	  unsigned short ushort=row["myunsignedshort"].data<unsigned short>();
	  if(ushort!=USHRT_MAX){
	    std::cout<<"wrong ushort "<<ushort<<std::endl;
	  }
	  long myl=row["mylong"].data<long>();
	  if(myl!=LONG_MAX){
	    std::cout<<"wrong myl "<<myl<<std::endl;
	  }
	  unsigned long myul=row["myunsignedlong"].data<unsigned long>();
	  if(myul!=ULONG_MAX){
	    std::cout<<"wrong myul "<<myul<<std::endl;
	  }
	  seal::IntBits<64>::SLeast mysll=row["mysleast"].data< seal::IntBits<64>::SLeast >();
	  if(mysll!=LONG_LONG_MAX){
	    std::cout<<"wrong mysll "<<mysll<<std::endl;
	  }
	  seal::IntBits<64>::ULeast myull=row["myuleast"].data< seal::IntBits<64>::ULeast >();
	  if(myull!=ULONG_LONG_MAX){
	    std::cout<<"wrong myull "<<myull<<std::endl;
	  }
	  bool mybool=row["mybool"].data<bool>();
	  if(mybool!=false){
	    std::cout<<"wrong mybool "<<mybool<<std::endl;
	  }
	  char mychar=row["mychar"].data<char>();
	  if(mychar!=CHAR_MAX){
	    std::cout<<"wrong mychar "<<mychar<<std::endl;
	  }
	  unsigned char myuchar=row["myuschar"].data<unsigned char>();
	  if(myuchar!=UCHAR_MAX){
	    std::cout<<"wrong myuchar "<<myuchar<<std::endl;
	  }
	}
	++counter;
      }
      cursor.close();
       // Committing the transaction
      std::cout << "Committing..." << std::endl;
      session->transaction().commit(); 
      session->endUserSession();
      delete session;
      session=0;
      // Disconnecting
      std::cout << "Disconnecting..." << std::endl;
      connection->disconnect();
      delete connection;
  }catch ( coral::Exception& se ) {
    std::cerr << "CORAL Exception : " << se.what() << std::endl;
  }catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
  }catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
  }
  std::cout << "Exiting..." << std::endl;
}

