#include <iostream>
#include <stdexcept>
//#include <memory>
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/IColumn.h"
#include "RelationalAccess/IPrimaryKey.h"
#include "RelationalAccess/IForeignKey.h"
#include "RelationalAccess/IIndex.h"
#include "RelationalAccess/ITablePrivilegeManager.h"
#include "RelationalAccess/TableDescription.h"
#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/Exception.h"
#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/RelationalServiceException.h"
#include "PluginManager/PluginManager.h"
#include "SealKernel/Context.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/MessageStream.h"
int main(){
  std::string con="sqlite_file:zhen.db";
  seal::Handle< seal::Context > context=new seal::Context;
  seal::PluginManager* pm = seal::PluginManager::get();
  pm->initialise();
  try {
    seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader(context.get());
    loader->load( "SEAL/Services/MessageService" );
    std::vector< seal::Handle<seal::IMessageService> > v_msgSvc;
    context->query( v_msgSvc );
    if ( ! v_msgSvc.empty() ) {
      seal::Handle<seal::IMessageService>& msgSvc = v_msgSvc.front();
      msgSvc->setOutputLevel( seal::Msg::Debug );
    }
    loader->load( "CORAL/RelationalPlugins/sqlite" );
    seal::IHandle<coral::IRelationalDomain> iHandle=context->query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/sqlite" );
    if ( ! iHandle ) {
      throw coral::NonExistingDomainException( "sqlite" );
    }
    std::pair<std::string, std::string> connectionAndSchema = iHandle->decodeUserConnectionString( con );
    coral::IConnection*  connection=iHandle->newConnection( connectionAndSchema.first );
    connection->connect();
    coral::ISession* session=connection->newSession(connectionAndSchema.second);  
    std::string userName, password;
    session->startUserSession( userName, password );
    session->transaction().start(false);
    coral::ISchema& schema = session->nominalSchema();
    schema.dropIfExistsTable( "mytest" );
    coral::TableDescription description0;
    description0.setName( "mytest" );
    description0.insertColumn( "ID",coral::AttributeSpecification::typeNameForId( typeid(int) ) );
    description0.insertColumn( "X",coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    description0.insertColumn( "Y",coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    description0.insertColumn( "ORDER",coral::AttributeSpecification::typeNameForId( typeid(int) ) );
    std::vector<std::string> idx_cols;
    idx_cols.push_back("ORDER");
    description0.createIndex("IDX1",idx_cols,false);
    schema.createTable( description0 );
    std::set<std::string> result=schema.listTables();
    for(std::set<std::string>::iterator it=result.begin(); it!=result.end(); ++it){
      std::cout<<"table names: "<<*it<<std::endl;
      schema.dropTable(*it);
    }    
    //Committing the transaction
    std::cout << "Committing..." << std::endl;
    session->transaction().commit();
    session->transaction().start();
    std::set<std::string> r=schema.listTables();
    for(std::set<std::string>::iterator it=r.begin(); it!=r.end(); ++it){
      std::cout<<"table names after drop: "<<*it<<std::endl;
    }    
    std::cout << "Committing..." << std::endl;
    session->transaction().commit();
    session->endUserSession();
    delete session;
    session=0;
    // Disconnecting
    std::cout << "Disconnecting..." << std::endl;
    connection->disconnect();
    delete connection;
  }catch ( coral::Exception& se ) {
    std::cerr << "CORAL Exception : " << se.what() << std::endl;
  }catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
  }catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
  }
}
