#include "RelationalAccess/ISession.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/IBulkOperation.h"
#include "RelationalAccess/IQuery.h"
#include "RelationalAccess/ICursor.h"
#include "RelationalAccess/IRelationalDomain.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/Date.h"
#include "CoralBase/Exception.h"
#include "CoralBase/TimeStamp.h"
#include "SealBase/TimeInfo.h"
#include "RelationalAccess/RelationalServiceException.h"
#include "PluginManager/PluginManager.h"
#include "SealKernel/Context.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/MessageStream.h"
#include <iostream>
#include <stdexcept>
int main(){
  std::string con="sqlite_file:zhen.db";
  seal::Handle< seal::Context > context=new seal::Context;
  seal::PluginManager* pm = seal::PluginManager::get();
  pm->initialise();
  try{
     seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader(context.get());
    loader->load( "SEAL/Services/MessageService" );
    std::vector< seal::Handle<seal::IMessageService> > v_msgSvc;
    context->query( v_msgSvc );
    if ( ! v_msgSvc.empty() ) {
      seal::Handle<seal::IMessageService>& msgSvc = v_msgSvc.front();
      msgSvc->setOutputLevel( seal::Msg::Debug );
    }
    loader->load( "CORAL/RelationalPlugins/sqlite" );
    seal::IHandle<coral::IRelationalDomain> iHandle=context->query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/sqlite" );
    if ( ! iHandle ) {
      throw coral::NonExistingDomainException( "sqlite" );
    }
    std::pair<std::string, std::string> connectionAndSchema = iHandle->decodeUserConnectionString( con );
    coral::IConnection*  connection=iHandle->newConnection( connectionAndSchema.first );
    connection->connect();
    coral::ISession* session=connection->newSession(connectionAndSchema.second);  
    std::string userName, password;
    session->startUserSession( userName, password );
    session->transaction().start(false);
    coral::ISchema& schema = session->nominalSchema();
    std::cout << "About to drop previous table" << std::endl;
    schema.dropIfExistsTable( "T" );    
    std::cout << "Describing new table" << std::endl;
    coral::TableDescription description;
    description.setName( "T" );
    description.insertColumn( "ID",
			      coral::AttributeSpecification::typeNameForId( typeid(int) ) );
    description.setPrimaryKey( "ID" );
    description.insertColumn( "TheDate",
			      coral::AttributeSpecification::typeNameForId( typeid(coral::Date) ) );
    description.insertColumn( "TheTime",
			      coral::AttributeSpecification::typeNameForId( typeid(coral::TimeStamp) ),
			      6 );
    
    std::cout << "About to create the table" << std::endl;
    coral::ITable& table = schema.createTable( description );
    coral::AttributeList rowBuffer;
    table.dataEditor().rowBuffer( rowBuffer );
    
    for ( int i = 0; i < 5; ++i )
      {
	rowBuffer["ID"].data<int>() = i;
	rowBuffer["TheDate"].data<coral::Date>() = coral::Date( 2005, 11, i+1 );
	rowBuffer["TheTime"].data<coral::TimeStamp>() = coral::TimeStamp::now();
	table.dataEditor().insertRow( rowBuffer );
      }
    coral::IBulkOperation* bulkInserter = table.dataEditor().bulkInsert( rowBuffer, 3 );
    
    long fraction = 111111111;
    for ( int i = 5; i < 10; ++i, fraction /= 10 )
      {
	rowBuffer["ID"].data<int>() = i;
	rowBuffer["TheDate"].data<coral::Date>() = coral::Date( 2005, 11, i+1 );
	coral::TimeStamp thisMoment = coral::TimeStamp::now();
	rowBuffer["TheTime"].data<coral::TimeStamp>() = coral::TimeStamp( thisMoment.year(),
									  thisMoment.month(),
									  thisMoment.day(),
									  thisMoment.hour(),
									  thisMoment.minute(),
									  thisMoment.second(),
									  (i-4) * fraction );
	bulkInserter->processNextIteration();
      }
    bulkInserter->flush();
    delete bulkInserter;
    session->transaction().commit();
    std::cout<<"table created"<<std::endl;
    session->transaction().start( true );
    schema = session->nominalSchema();
    coral::IQuery* query = schema.tableHandle("T").newQuery();
    query->setRowCacheSize( 20 );
    coral::ICursor& cursor = query->execute();
    int i = 0;
    while ( cursor.next() ) {
      const coral::AttributeList& currentRow = cursor.currentRow();
      currentRow.toOutputStream( std::cout ) << std::endl;
      ++i;
    }
    if ( i != 10 )
      throw std::runtime_error( "Unexpected number of rows" );
    delete query;
    session->transaction().commit();
    session->endUserSession();
    delete session;
    session=0;
    // Disconnecting
    std::cout << "Disconnecting..." << std::endl;
    connection->disconnect();
    delete connection;
  }catch ( coral::Exception& se ) {
    std::cerr << "CORAL Exception : " << se.what() << std::endl;
    return 1;
  }catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return 1;
  }catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
    return 1;
  }
  return 0;
}
