#include <iostream>
#include <stdexcept>
#include "RelationalAccess/RelationalServiceException.h"
#include "RelationalAccess/SchemaException.h"
#include "RelationalAccess/IRelationalDomain.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/IConnection.h"
#include "RelationalAccess/ITransaction.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITable.h"
#include "RelationalAccess/ITableSchemaEditor.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/IPrimaryKey.h"
#include "RelationalAccess/IForeignKey.h"
#include "RelationalAccess/IIndex.h"
#include "RelationalAccess/ITableDataEditor.h"
#include "CoralBase/AttributeList.h"
#include "CoralBase/Attribute.h"
#include "CoralBase/AttributeSpecification.h"
#include "CoralBase/Exception.h"
#include "PluginManager/PluginManager.h"
#include "SealKernel/Context.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/MessageStream.h"
int main( int, char** )
{
  std::string con = "sqlite_file:zhen.db";
  seal::Handle< seal::Context > context=new seal::Context;
  seal::PluginManager* pm = seal::PluginManager::get();
  pm->initialise();
  try {
    seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader(context.get());
    loader->load( "SEAL/Services/MessageService" );
    std::vector< seal::Handle<seal::IMessageService> > v_msgSvc;
    context->query( v_msgSvc );
    if ( ! v_msgSvc.empty() ) {
      seal::Handle<seal::IMessageService>& msgSvc = v_msgSvc.front();
      msgSvc->setOutputLevel( seal::Msg::Debug );
    }
    loader->load( "CORAL/RelationalPlugins/sqlite" );
    seal::IHandle<coral::IRelationalDomain> iHandle=context->query<coral::IRelationalDomain>( "CORAL/RelationalPlugins/sqlite" );
    if ( ! iHandle ) {
      throw coral::NonExistingDomainException( "sqlite" );
    }
    
    std::pair<std::string, std::string> connectionAndSchema = iHandle->decodeUserConnectionString( con );
    coral::IConnection*  connection=iHandle->newConnection( connectionAndSchema.first );
    connection->connect();
    coral::ISession* session=connection->newSession(connectionAndSchema.second);  
    std::string userName, password;
    session->startUserSession( userName, password );
    session->transaction().start(false);
    //
    // create table t1
    //
    session->nominalSchema().dropIfExistsTable( "t1" );
    std::cout<<"creating table t1"<<std::endl;
    coral::TableDescription desc1;
    desc1.setName("t1");
    desc1.insertColumn( "id",coral::AttributeSpecification::typeNameForId( typeid(int) ) );
    desc1.insertColumn( "first", coral::AttributeSpecification::typeNameForId(typeid(std::string) ) );
    desc1.insertColumn( "datacol", coral::AttributeSpecification::typeNameForId(typeid(std::string) ) );
    desc1.setPrimaryKey( "datacol", "" );
    coral::ITable& t1 = session->nominalSchema().createTable(desc1);
    std::cout<<"table t1 created"<<std::endl;

    //
    // create table t2
    //
    std::cout<<"creating table t2"<<std::endl;
    session->nominalSchema().dropIfExistsTable( "t2" );
    coral::TableDescription desc2;
    desc2.setName("t2");
    desc2.insertColumn( "id2", coral::AttributeSpecification::typeNameForId( typeid(int) ) );
    desc2.insertColumn( "col1", coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    desc2.insertColumn( "col2", coral::AttributeSpecification::typeNameForId( typeid(std::string) ) );
    desc2.insertColumn( "col3", coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    desc2.insertColumn( "col4", coral::AttributeSpecification::typeNameForId( typeid(std::string) ) );
    desc2.insertColumn( "col5", coral::AttributeSpecification::typeNameForId( typeid(float) ) );
    bool isUnique=false;
    std::vector<std::string> nonuniqueidx;
    nonuniqueidx.push_back("col3");
    nonuniqueidx.push_back("col4");
    desc2.createIndex( "t2_idxNU", nonuniqueidx ,isUnique);
    isUnique=true;
    std::vector<std::string> uniqueidx;
    uniqueidx.push_back("col1");
    uniqueidx.push_back("col5");
    desc2.createIndex( "t2_idxU", uniqueidx ,isUnique);
    coral::ITable& t2 = session->nominalSchema().createTable( desc2 );    
    session->transaction().commit();
    std::cout<<"table t2 created"<<std::endl;
    //
    // change t1 schema
    //
    session->transaction().start(false);
    std::cout<<"adding column to t1"<<std::endl;
    t1.schemaEditor().insertColumn( "extra", coral::AttributeSpecification::typeNameForId(typeid(short) ) );
    std::cout<<"setting null constraint to first"<<std::endl;
    t1.schemaEditor().setNotNullConstraint( "first" );
    std::cout<<"setting unique constraint to first"<<std::endl;
    t1.schemaEditor().setUniqueConstraint( "first" );
    std::cout<<"change column type id"<<std::endl;
    t1.schemaEditor().changeColumnType("id",coral::AttributeSpecification::typeNameForId(typeid(unsigned int) ) );
    session->transaction().commit();
    std::cout<<"column added to t1"<<std::endl;
    
    //
    // change t2 schema
    //
    session->transaction().start(false);
    std::cout<<"[OVAL] rename column from t2"<<std::endl;
    t2.schemaEditor().renameColumn("col1","newcol1");
    std::cout<<"[OVAL] drop column from t2"<<std::endl;
    t2.schemaEditor().dropColumn( "col2" );
    std::cout<<"[OVAL] insert column col2 back to t2"<<std::endl;
    t2.schemaEditor().insertColumn( "col2", coral::AttributeSpecification::typeNameForId(typeid(int) ) );
    session->transaction().commit();
    std::cout<<"[OVAL] column changed for t2"<<std::endl;
    session->endUserSession();
    delete session;
    session=0;
    // Disconnecting
    std::cout << "Disconnecting..." << std::endl;
    connection->disconnect();
    delete connection;
  }catch ( coral::SchemaException& ce){
    std::cerr <<"caught coral Exception : "<< ce.what() <<std::endl;
  }catch ( coral::Exception& se ) {
    std::cerr << "CORAL Exception : " << se.what() << std::endl;
    return 1;
  }catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return 1;
  }catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
    return 1;
  }
  return 0;
}
