#include "TestApp.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/IMessageService.h"
#include "SealKernel/PropertyManager.h"
#include "PluginManager/PluginManager.h"
#include <stdexcept>
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITransaction.h"
#include "CoralBase/Exception.h"
#include "CoralBase/AttributeSpecification.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"
#include "../../../ConnectionService/src/ConnectionService.h"
#include "../../../ConnectionService/src/SessionProxy.h"
#include "../../../ConnectionService/src/ConnectionPool.h"

const std::string TestApp::OVALTAG = "[OVAL]";

TestApp::TestApp():
  m_context( new seal::Context ){
  this->loadServices();
}


TestApp::~TestApp()
{
}

void
TestApp::loadServices()
{
  seal::PluginManager* pm = seal::PluginManager::get();
  pm->initialise();
  seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( m_context.get() );

  // Load the SEAL Message service and set the output threshold to Info
  loader->load( "SEAL/Services/MessageService" );
  std::vector< seal::Handle<seal::IMessageService> > v_msgSvc;
  m_context->query( v_msgSvc );
  if ( ! v_msgSvc.empty() ) {
    seal::Handle<seal::IMessageService>& msgSvc = v_msgSvc.front();
    msgSvc->setOutputLevel( seal::Msg::Debug);
  }

  // Load the Connection Service
  std::cout << "Loading Connection Service"<<std::endl;
  loader->load( "CORAL/Services/ConnectionService" );
}

seal::IHandle<coral::IConnectionService>
TestApp::connectionService()
{
  std::vector< seal::IHandle<coral::IConnectionService> > v_svc;
  m_context->query( v_svc );
  if ( v_svc.empty() ) {
    throw std::runtime_error( "Could not locate the connection service" );
  }
  return v_svc.front();
}

void
TestApp::run(){
  
  seal::IHandle<coral::IConnectionService> theConnectionService = connectionService();
  if(! theConnectionService ){
    std::cout << "Connection Service not loaded."<<std::endl;
    return;
  }

  try {
    theConnectionService->configuration().setConnectionTimeOut(0);
    std::cout << "Getting the first sessionProxy object "<<std::endl;
    std::string connectionString("/dbdev/service_1");  
    coral::ISessionProxy* session1 = theConnectionService->connect( connectionString );
    session1->transaction().start( true );
    std::set<std::string> tables = session1->nominalSchema().listTables();
    session1->transaction().commit();
    if(tables.find("T_CONNECTION_SERVICE_TEST")==tables.end()){
      std::cout << OVALTAG <<"TEST ERROR with connection "<<connectionString<<" table T_CONNECTION_SERVICE_TEST is not found."<<std::endl;
    } else {
      std::cout << OVALTAG << "Connection "<<connectionString<<" was ok."<<std::endl;
    }
    coral::ConnectionService::ConnectionService* myService =
      dynamic_cast<coral::ConnectionService::ConnectionService*>(theConnectionService.get());
    
    coral::ISessionProxy* session2 = theConnectionService->connect( connectionString );
    coral::ISessionProxy* session3 = theConnectionService->connect( connectionString );
    coral::ISessionProxy* session4 = theConnectionService->connect( connectionString );
    coral::ISessionProxy* session5 = theConnectionService->connect( connectionString );
    std::cout << "--------------------------(1)"<<std::endl;
    std::cout << OVALTAG <<"Number of idle connection:"<< myService->numberOfIdleConnectionsInPool()<<std::endl;
    std::cout << OVALTAG <<"Number of active connection:"<< myService->numberOfActiveConnectionsInPool()<<std::endl;  
    delete session1;
    delete session2;
    delete session3;
    delete session4;
    delete session5;
    std::cout << "--------------------------(2)"<<std::endl;
    std::cout << OVALTAG <<"Number of idle connection:"<< myService->numberOfIdleConnectionsInPool()<<std::endl;
    std::cout << OVALTAG <<"Number of active connection:"<< myService->numberOfActiveConnectionsInPool()<<std::endl;  
    theConnectionService->purgeConnectionPool();
    std::cout << "--------------------------(3)"<<std::endl;
    std::cout << OVALTAG <<"Number of idle connection:"<< myService->numberOfIdleConnectionsInPool()<<std::endl;
    std::cout << OVALTAG <<"Number of active connection:"<< myService->numberOfActiveConnectionsInPool()<<std::endl;  
  }catch ( const coral::Exception& exc) {
    std::cout << "TEST ERROR: "<<exc.what()<<std::endl;
  }
  
   
}
