#include "TestApp.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/IMessageService.h"
#include "PluginManager/PluginManager.h"
#include <stdexcept>
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITypeConverter.h"
#include "RelationalAccess/TableDescription.h"
#include "RelationalAccess/ISchema.h"
#include "RelationalAccess/ITransaction.h"
#include "CoralBase/Exception.h"
#include "CoralBase/AttributeSpecification.h"
#include "RelationalAccess/ISession.h"
#include "RelationalAccess/ISessionProxy.h"
#include "RelationalAccess/IConnectionService.h"
#include "RelationalAccess/IConnectionServiceConfiguration.h"
#include "CoralCommon/Utilities.h"
#include "CoralBase/AttributeSpecification.h"

const std::string TestApp::OVALTAG = "[OVAL]";

TestApp::TestApp():
  m_context( new seal::Context ){
  this->loadServices();
}


TestApp::~TestApp()
{
}

void
TestApp::loadServices()
{
  seal::PluginManager* pm = seal::PluginManager::get();
  pm->initialise();
  seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( m_context.get() );

  // Load the SEAL Message service and set the output threshold to Info
  loader->load( "SEAL/Services/MessageService" );
  std::vector< seal::Handle<seal::IMessageService> > v_msgSvc;
  m_context->query( v_msgSvc );
  if ( ! v_msgSvc.empty() ) {
    seal::Handle<seal::IMessageService>& msgSvc = v_msgSvc.front();
    msgSvc->setOutputLevel( seal::Msg::Debug);
  }

  // Load the Connection Service
  std::cout << "Loading Connection Service"<<std::endl;
  loader->load( "CORAL/Services/ConnectionService" );
}

seal::IHandle<coral::IConnectionService>
TestApp::connectionService()
{
  std::vector< seal::IHandle<coral::IConnectionService> > v_svc;
  m_context->query( v_svc );
  if ( v_svc.empty() ) {
    throw std::runtime_error( "Could not locate the connection service" );
  }
  return v_svc.front();
}

void
TestApp::run(){
  
  
  seal::IHandle<coral::IConnectionService> theConnectionService = connectionService();
  if(! theConnectionService ){
    std::cout << "Connection Service not loaded."<<std::endl;
    return;
  }
  //theConnectionService->configuration().setConnectionTimeOut(10);
  //theConnectionService->purgeConnectionPool();

  std::string connectionString("sqlite_file:mydb.db");
  coral::ISessionProxy* session = 0;
  
  try {

    session = theConnectionService->connect( connectionString, coral::Update  );
    coral::ITransaction& transaction = session->transaction();
    bool active = transaction.isActive();
    if(!active) transaction.start(true);

//unused?    coral::ISchema& nominalSchema = session->nominalSchema(); 
   
    if(!active) transaction.commit();
    std::cout << OVALTAG << "Connection "<<connectionString<<" was ok."<<std::endl;
    

    coral::ISessionProxy* tmpSession = theConnectionService->connect( connectionString, coral::ReadOnly  );
    coral::ITransaction& tmptransaction = tmpSession->transaction();
    bool tmpactive = tmptransaction.isActive();    
    if(!tmpactive) tmptransaction.start( true );

    if (tmpSession->nominalSchema().existsTable("MyTABLE"))
      std::cout << "found MyTable." << std::endl;
    

    if(!tmpactive) tmptransaction.commit();
    std::cout << OVALTAG << "Connection ok"<<connectionString<<" was ok."<<std::endl;
    delete tmpSession;
 
    if(!active) transaction.start(true);
    //session->transaction().start(true);
    coral::ITypeConverter& typeConverter = session->typeConverter();
    std::string oldSqlTypeForString = typeConverter.sqlTypeForCppType( coral::AttributeSpecification::typeNameForType<std::string>() );
    coral::TableDescription descr;
    descr.setName("T_CONNECTION_SERVICE_TEST");
    descr.insertColumn("N_X",coral::AttributeSpecification::typeNameForType<int>());
    descr.insertColumn("N_S",coral::AttributeSpecification::typeNameForType<std::string>());
    session->nominalSchema().createTable( descr );
    if(!active) transaction.commit();
    //session->transaction().commit();


  }catch ( const coral::Exception& exc) {
   std::cout << "un expected ERROR: "<<exc.what()<<std::endl;
  }
    
   
}
