#include <iostream>
#include <sstream>
#include <exception>
#include "CoralBase/Exception.h"
#include "TestDriver.h"
#include <cstdlib>

int main( int, char** )
{
  try {
    if ( ! ::getenv("CORAL_AUTH_PATH") )
      ::putenv( "CORAL_AUTH_PATH=/afs/cern.ch/sw/lcg/app/pool/db" );
    if ( ! ::getenv("CORAL_DBLOOKUP_PATH") )
      ::putenv( "CORAL_DBLOOKUP_PATH=/afs/cern.ch/sw/lcg/app/pool/db" );

    TestDriver testDriver;

    for ( int i = 0; i < 3; ++i ) {

      std::ostringstream os;
      os << "/dbdev/service_" << i + 1;
      std::string serviceName = os.str();

      std::cout << "    preparing the schema in " << serviceName << "..." << std::endl;
      testDriver.prepareSchema( serviceName );

      std::cout << "    simple row inserts in " << serviceName << "..." << std::endl;
      testDriver.writeSimple( serviceName );

      std::cout << "    bulk row inserts in " << serviceName << "..." << std::endl;
      testDriver.writeBulk( serviceName );

      std::cout << "    reading back the data from " << serviceName << "..." << std::endl;
      testDriver.read( serviceName );

      std::cout << "[OVAL] Tests for database service " << serviceName << " passed." << std::endl;
    }

  }
  catch ( coral::Exception& e ) {
    std::cerr << "CORAL Exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( std::exception& e ) {
    std::cerr << "C++ Exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Unhandled exception " << std::endl;
    return 1;
  }
  std::cout << "[OVAL] Success..." << std::endl;
  return 0;
}
