#include "Connectivity/ApplicationBase.h"

#include "RelationalAccess/IConnectionService.h"

#include "SealKernel/Context.h"
#include "SealKernel/ComponentLoader.h"
#include "SealKernel/IMessageService.h"
#include "PluginManager/PluginManager.h"

#include <iostream>
#include <cstdlib>

coral::Testing::ApplicationBase::ApplicationBase():
  m_context( new seal::Context )
{

  if ( ! ::getenv( "CORAL_AUTH_PATH" ) ) {
    ::putenv( "CORAL_AUTH_PATH=/afs/cern.ch/sw/lcg/app/pool/db/connectivity_tests" );
  }

  if ( ! ::getenv( "CORAL_DBLOOKUP_PATH" ) ) {
    ::putenv( "CORAL_DBLOOKUP_PATH=/afs/cern.ch/sw/lcg/app/pool/db/connectivity_tests" );
  }

  seal::PluginManager* pm = seal::PluginManager::get();
  pm->initialise();
  seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( m_context.get() );

  loader->load( "SEAL/Services/MessageService" );
  std::vector< seal::Handle<seal::IMessageService> > v_msgSvc;
  m_context->query( v_msgSvc );
  if ( ! v_msgSvc.empty() ) {
    seal::Handle<seal::IMessageService>& msgSvc = v_msgSvc.front();
    msgSvc->setOutputLevel( seal::Msg::Info);
  }

  loader->load( "CORAL/Services/ConnectionService" );
}

coral::Testing::ApplicationBase::~ApplicationBase()
{
  m_context = 0;
}


coral::IConnectionService&
coral::Testing::ApplicationBase::connectionService()
{
  seal::IHandle<coral::IConnectionService> iHandle =
    m_context->query<coral::IConnectionService>( "CORAL/Services/ConnectionService" );
  return *iHandle;
}


void
coral::Testing::ApplicationBase::run()
{
  std::cout << "First read operation starts" << std::endl;
  this->read1();
  std::cout << "First read operation ended" << std::endl << std::endl;

  std::cout << "Type ENTER to continue to second read operation" << std::endl;
  std::cin.get();
  std::cout << "Second read operation starts" << std::endl;
  this->read2();
  std::cout << "Second read operation ended" << std::endl;
}


void
coral::Testing::ApplicationBase::read1()
{
}

void
coral::Testing::ApplicationBase::read2()
{
}
