#include "Reporter.h"
#include "GlobalStatus.h"
#include "TopThread.h"
#include "RelationalAccess/ConnectionService.h"
#include "CoralBase/Exception.h"
#include <boost/thread.hpp>
#include <iostream>
#include <vector>
#include <stdexcept>
#include <cstdlib>


int main( int, char** )
{
  if ( ! ::getenv( "CORAL_AUTH_PATH" ) ) {
    ::putenv( "CORAL_AUTH_PATH=/afs/cern.ch/sw/lcg/app/pool/db" );
  }

  if ( ! ::getenv( "CORAL_DBLOOKUP_PATH" ) ) {
    ::putenv( "CORAL_DBLOOKUP_PATH=/afs/cern.ch/sw/lcg/app/pool/db" );
  }

  try {
    coral::ConnectionService connectionService;
    connectionService.setMessageVerbosityLevel( seal::Msg::Warning );
    Reporter reporter;

    GlobalStatus ok;
    std::vector< boost::thread* > threads;
    for ( int i = 0; i < 10; ++i )
      threads.push_back( new boost::thread( TopThread( connectionService, reporter, i + 1, ok ) ) );
    
    for ( int i = 0; i < 10; ++i )
      threads[i]->join();
    
    for ( int i = 0; i < 10; ++i ) {
      delete threads[i];
    }

    if ( ! ( ok.isOk() ) )
      throw std::runtime_error( "Test failed!" );
  }
  catch ( coral::Exception& e ) {
    std::cerr << "CORAL Exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( std::exception& e ) {
    std::cerr << "C++ Standard Exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Unhandled exception ..." << std::endl;
    return 1;
  }
  std::cout << "[OVAL] Test passed!" << std::endl;
  return 0;
}
