#include <iostream>
#include <vector>
#include <stdexcept>
#include "PluginManager/PluginManager.h"
#include "SealKernel/ComponentLoader.h"
#include "CoralBase/Exception.h"
#include "SealKernel/Context.h"
#include "SealKernel/Service.h"
#include "SealKernel/PropertyManager.h"
#include "RelationalAccess/IAuthenticationService.h"
#include "RelationalAccess/IAuthenticationCredentials.h"
#include "RelationalAccess/AuthenticationServiceException.h"

int main( int, char** )
{
  try {
    std::vector< std::string > connections;
    connections.push_back( "oracle://devdb9/IOANNIS" );
    connections.push_back( "oracle://devdb10/IOANNIS" );

    seal::PluginManager* pm = seal::PluginManager::get();
    pm->initialise();

    seal::Handle< seal::Context > context = new seal::Context();
    seal::Handle<seal::ComponentLoader> loader = new seal::ComponentLoader( context.get() );

    loader->load( "CORAL/Services/XMLAuthenticationService" );

    seal::IHandle<seal::Service> svc = context->query<seal::Service>( "CORAL/Services/XMLAuthenticationService" );

    seal::Handle<seal::PropertyManager>& pmgr = svc->propertyManager();
    seal::PropertyBase* property = pmgr->property( "AuthenticationFile" );
    property->set( std::string( "authentication.xml" ) );

    std::vector< seal::IHandle<coral::IAuthenticationService> > v_authSvc;
    context->query( v_authSvc );
    if ( ! v_authSvc.empty() ) {
      seal::IHandle<coral::IAuthenticationService>& authSvc = v_authSvc.front();

      for ( std::vector< std::string >::const_iterator iConnection = connections.begin();
            iConnection != connections.end(); ++iConnection ) {
        std::cout << "Connection \"" << *iConnection << "\" :" << std::endl;
        const coral::IAuthenticationCredentials& credentials = authSvc->credentials( *iConnection );
        
        std::cout << "  default role : " << std::endl;
        int numberOfItems = credentials.numberOfItems();
        for ( int i = 0; i < numberOfItems; ++i ) {
          std::string itemName = credentials.itemName( i );
          std::cout << "    " << itemName << " : " << credentials.valueForItem( itemName ) << std::endl;
        }

        std::string role = "writer";
        try {
          std::cout << "  role \"" << role << "\" : " << std::endl;
          const coral::IAuthenticationCredentials& credentials = authSvc->credentials( *iConnection, role );
          if ( iConnection != connections.begin() )
            throw std::runtime_error( "Fetched invalid role, and UnknownRoleException was not thrown!" );
          int numberOfItems = credentials.numberOfItems();
          for ( int i = 0; i < numberOfItems; ++i ) {
            std::string itemName = credentials.itemName( i );
            std::cout << "    " << itemName << " : " << credentials.valueForItem( itemName ) << std::endl;
          }
        }
        catch( coral::UnknownRoleException& ) {
          std::cout << "    ...does not exist. Exception is has been thrown (and caught)" << std::endl;
          if ( iConnection == connections.begin() )
            throw std::runtime_error( "Received unexpected exception when retrieving a role" );
        }
      }
    }
    else {
      throw std::runtime_error("Could not retrieve an authentication service.");
    }

    std::cout << "Exiting..." << std::endl;
  }
  catch ( coral::Exception& se ) {
    std::cerr << "CORAL Exception : " << se.what() << std::endl;
    return 1;
  }
  catch ( std::exception& e ) {
    std::cerr << "Standard C++ exception : " << e.what() << std::endl;
    return 1;
  }
  catch ( ... ) {
    std::cerr << "Exception caught (...)" << std::endl;
    return 1;
  }
  std::cout << "[OVAL] Success" << std::endl;
  return 0;
}
