#!/usr/bin/env python

import sys
import os
import os.path
import string

def findHeaderFiles( inputFilePath ):
    headerFiles = []
    file = open( inputFilePath, 'r' )
    lines=file.readlines()
    for line in lines:
        templine = string.lstrip( line )
        if len(templine) > 0 and templine[0] == '#':
            templine = string.lstrip( templine[1:])
            if len(templine) > 7 and string.find( templine, 'include' ) == 0:
                templine = string.lstrip( templine[7:] )
                if len(templine) > 0: templine = templine[1:]
                index = string.find( templine, '"' )
                if index != -1:
                    headerFiles.append( templine[:index] )
    file.close()
    return headerFiles


def findDependencies( inputFile, inputDir, includeDirs, dependencies ):
    inputFilePath = os.path.join( inputDir, inputFile )
    if not os.path.exists( inputFilePath ): return dependencies
    if os.path.isdir( inputFilePath ) or os.path.ismount( inputFilePath ): return dependencies
    headerFiles = findHeaderFiles( inputFilePath )
    for headerFile in headerFiles:
        extraDeps = []
        headerFilePathRelative = os.path.join( inputDir, headerFile )
        if os.path.exists( headerFilePathRelative ):
            if headerFilePathRelative not in dependencies:
                dependencies.append( headerFilePathRelative )
                dirAndFile = os.path.split( headerFilePathRelative )
                extraDeps = findDependencies( dirAndFile[1], dirAndFile[0], includeDirs, dependencies )
        else:
            for includeDir in includeDirs:
                headerFilePath = os.path.join( includeDir, headerFile )
                if os.path.exists( headerFilePath ):
                    if headerFilePath not in dependencies:
                        dependencies.append( headerFilePath )
                        dirAndFile = os.path.split( headerFilePath )
                        extraDeps = findDependencies( dirAndFile[1], dirAndFile[0], includeDirs, dependencies )
                        break
        for extraDep in extraDeps:
            if extraDep not in dependencies:
                dependencies.append( extraDep )
    return dependencies


try:
    # Check the number of arguments
    if len(sys.argv) < 4:
        errorString = "Syntax : " + os.path.split( sys.argv[0] )[1] + " file.cpp file.d objectExtension [include directories]"
        raise Exception, errorString

    inputDir = os.path.split( sys.argv[1] )[0]
    inputFile = os.path.split( sys.argv[1] )[1]
    outputDir = os.path.split( sys.argv[2] )[0]
    dependencyFile = os.path.split( sys.argv[2] )[1]
    objectFile = dependencyFile[:string.rfind( dependencyFile, '.' ) + 1] + sys.argv[3]

    includeDirs = []
    for iDir in range( 4, len(sys.argv) ):
        includeDirs.append( sys.argv[iDir] )

    dependencies = []
    dependencies = findDependencies( inputFile, inputDir, includeDirs, dependencies )

    depFile = open( os.path.join( outputDir, dependencyFile ), 'w' )
    depFile.write( os.path.join( outputDir, objectFile ) + ' ' )
    depFile.write( os.path.join( outputDir, dependencyFile ) + ': ' )
    depFile.write( os.path.join( inputDir, inputFile ) )
    for dependency in dependencies:
        depFile.write( ' \\\n  ' + dependency )
    depFile.write( '\n' )
    depFile.close()

except Exception, e:
    sys.stderr.write( str(e) + '\n' )

