#!/usr/bin/env python

import sys
import os
import os.path
import string


# Lists the subdirectories of a given directory
def listSubDirs( curdir ):
    excludedDirNames = ( 'CVS', 'doc', 'config', 'contrib', 'ReleaseTools', 'Common', 'cmt' )
    dirContents = os.listdir( curdir )
    dirsToProcess = []
    for content in dirContents :
        if os.path.isdir( os.path.join( curdir, content ) ) :
            if content not in excludedDirNames :
                dirsToProcess.append( content )
    return dirsToProcess

# Returns the dependent directories
def listDependencies( curdir ):
    dependencies = []
    if 'BuildFile' in os.listdir( curdir ):
        dependenciesFound = processBuildFile( os.path.join( curdir, 'BuildFile' ) )
        for dependency in dependenciesFound:
            dependencies.append( dependency )
    subDirs = listSubDirs( curdir )
    for subDir in subDirs:
        subDirDependencies = listDependencies( os.path.join( curdir, subDir ) )
        for subDirDependency in subDirDependencies:
            if subDirDependency not in dependencies:
                dependencies.append( subDirDependency )
    return dependencies

# Processes a BuildFile and extracts the dependencies
def processBuildFile( path ):
    dependencies = []
    file = open( path, 'r' )
    b = 1
    while b:
        line = file.readline()
        if line == '': break
        i_use = string.find( line, 'use ' )
        if i_use != -1:
            temp = line[i_use+4:]
            i_name = string.find( temp, 'name' )
            if i_name != -1:
                tempname = temp[i_name+4:]
                while tempname[0] is ' ' or tempname[0] is '=':
                    tempname = tempname[1:]
                i_gt = string.find( tempname, '>' )
                if i_gt != -1:
                    name = tempname[:i_gt]
                    while name[len( name ) - 1] is ' ':
                        name = name[:len( name ) - 1]
                    dependencies.append(name)
    return dependencies


########### Main Script starts here ###################


try:
    # Check the number of arguments
    if len(sys.argv) < 3:
        errorString = "Syntax : " + os.path.split( sys.argv[0] )[1] + " localTop classPathDirectory"
        raise Exception, errorString

    # Find the subdirectories
    localTop = sys.argv[1]
    classDir = sys.argv[2]
    curdir = os.path.join( localTop, classDir )
    dirsToProcess = listSubDirs( curdir )

    # Check if a 'Libraries' directory is there:
    exists_Libraries = 0
    if 'Libraries' in dirsToProcess:
        exists_Libraries = 1
        dirsToProcess.remove( 'Libraries' )

    # Correct for the src part of the class path
    relSrcDir = ''
    if classDir is not 'src':
        relSrcDir = classDir[4:]
    for i in range( len(dirsToProcess) ):
        dirsToProcess[i] = os.path.join( relSrcDir, dirsToProcess[i] )
    
    # Find the dependencies among the directories
    dirsAndDependencies = {}
    for iDir in dirsToProcess:
        deps = listDependencies( os.path.join( localTop, 'src', iDir ) )
        topDirDeps = []
        for dep in deps:
            if dep in dirsToProcess:
                topDirDeps.append( dep )
        dirsAndDependencies[ iDir ] = topDirDeps

    # Order the directories according to their dependencies
    orderedDirs = []
    while len(orderedDirs) != len(dirsToProcess):
        for iDir in dirsToProcess:
            if iDir not in orderedDirs:
                deps = dirsAndDependencies[ iDir ]
                if len(deps) == 0:
                    orderedDirs.append( iDir )
        for iDir in dirsToProcess:
            if iDir not in orderedDirs:
                deps = dirsAndDependencies[ iDir ]
                for dep in orderedDirs:
                    if dep in deps:
                        deps.remove( dep )

    # Correct back the names
    for ioDir in range( len(orderedDirs) ):
        orderedDirs[ioDir] = os.path.split( orderedDirs[ioDir] )[1]

    # Print the correct order
    if exists_Libraries:
        sys.stdout.write( 'Libraries ' )
    for iDir in orderedDirs:
        sys.stdout.write( iDir + ' ' )
    sys.stdout.write( '\n' )

except Exception, e:
    sys.stderr.write( str(e) + '\n' )

